#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

import sys
import os
import inspect
import fnmatch
import datetime
# Local import
from realog import debug
from . import module
from . import tools
from . import env
##
## @brief System class represent the pre-build Module that are already install and accessible in the system environment
##
class System:
	##
	## @brief Constructor
	## @param[in] self (handle) Class handle
	## @return None
	##
	def __init__(self):
		self._valid=False;
		self._help="";
		self._export_depends=[]
		self._export_flags={}
		self._export_src=[]
		self._export_path=[]
		self._action_on_state={}
		self._headers=[]
		self._version=None
	
	##
	## @brief Set the help of this system Module
	## @param[in] self (handle) Class handle
	## @param[in] help (string) Help for the user
	## @return None
	##
	def set_help(self, help):
		self._help = help;
	
	##
	## @brief Get the help of this system Module
	## @param[in] self (handle) Class handle
	## @return (string) Help for the user
	##
	def get_help(self):
		return self._help;
	
	##
	## @brief Set validity state of the system Module
	## @param[in] self (handle) Class handle
	## @param[in] state (bool) New valididty state of the system module
	## @return None
	##
	def set_valid(self, state):
		self._valid = state
	
	##
	## @brief Get validity state of the system Module
	## @param[in] self (handle) Class handle
	## @return (bool) New valididty state of the system module
	##
	def get_valid(self):
		return self._valid
	
	##
	## @brief Add source element
	## @param[in] self (handle) Class handle
	## @param[in] list ([string,...]) List of all Files to add. ex: *.a, *.so ...
	## @return None
	##
	def add_sources(self, list):
		tools.list_append_to(self._export_src, list)
	
	##
	## @brief Add include path of the sources
	## @param[in] self (handle) Class handle
	## @param[in] list ([string,...]) List of all path to add in the -I include element
	## @return None
	##
	# todo : add other than C ...
	def add_path(self, list):
		tools.list_append_to(self._export_path, list)
	
	##
	## @brief Add a dependency on this module
	## @param[in] self (handle) Class handle
	## @param[in] list ([string,...] or string) Name(s) of the modules dependency
	## @return None
	##
	def add_depend(self, list):
		tools.list_append_to(self._export_depends, list, True)
	
	##
	## @brief Add compilation flags
	## @param[in] self (handle) Class handle
	## @param[in] type (string) inclusion group name 'c', 'c++', 'java' ...
	## @param[in] list ([string,...] or string) List of path to include
	## @return None
	##
	def add_flag(self, type, list):
		tools.list_append_to_2(self._export_flags, type, list)
	
	##
	## @brief Set version of the module
	## @param[in] self (handle) Class handle
	## @param[in] version_list ([int,...]) Ids of the version. ex: [1,2,5] or [0,8,"dev"]
	## @return None
	##
	def set_version(self, version_list):
		self._version = version_list
	
	## @copydoc lutin.module.Target.add_action
	def add_action(self, name_of_state="PACKAGE", level=5, name="no-name", action=None):
		if name_of_state not in self._action_on_state:
			self._action_on_state[name_of_state] = [[level, name, action]]
		else:
			self._action_on_state[name_of_state].append([level, name, action])
	
	## @copydoc lutin.module.Module.add_header_file
	def add_header_file(self, list, destination_path=None, clip_path=None, recursive=False):
		self._headers.append({
		  "list":list,
		  "dst":destination_path,
		  "clip":clip_path,
		  "recursive":recursive
		  })
	##
	## @brief Generate a string representing the class (for str(xxx))
	## @param[in] self (handle) Class handle
	## @return (string) string of str() convertion
	##
	def __repr__(self):
		return "{lutin.System}"
	
	##
	## @brief Configure a module with internal datas
	## @param[in] self (handle) Class handle
	## @param[in] target (handle) @ref lutin.module.Target handle
	## @param[in] module (handle) @ref lutin.module.Module handle
	## @return None
	##
	def configure_module(self, target, module):
		# add element flags to export
		for elem in self._export_flags:
			debug.verbose("add element :" + str(elem) + " elems=" + str(self._export_flags[elem]))
			module.add_flag(elem, self._export_flags[elem], export=True)
		# add module dependency
		if self._export_depends != []:
			module.add_depend(self._export_depends)
		# add exporting sources
		if self._export_src != []:
			module.add_src_file(self._export_src)
		# add export path
		if self._export_path != []:
			# no control on API
			module._add_path(self._export_path, export=True)
		# Export all actions ...
		for elem in self._action_on_state:
			level, name, action = self._action_on_state[elem]
			target.add_action(elem, level, name, action)
		for elem in self._headers:
			module.add_header_file(
			    elem["list"],
			    destination_path=elem["dst"],
			    clip_path=elem["clip"],
			    recursive=elem["recursive"])
		if self._version != None:
			module.set_pkg("VERSION", self._version);
		


##
## @brief Create a @ref lutin.module.Module for the system list elements
## @param[in] target (handle) @ref lutin.target.Target handle
## @param[in] name (string) Name of the system module
##
def create_module_from_system(target, dict):
	my_module = module.Module(dict["path"], dict["name"], 'PREBUILD')
	dict["system"].configure_module(target, my_module)
	return my_module




# Dictionnaire of Target name
#   inside table of ["Name of the lib", "path of the lib", boolean loaded, module loaded]
__system_list={}
__start_system_name="System_"

##
## @brief Import all File that start with env.get_build_system_base_name() + __start_system_name + XXX and register in the list of System
## @param[in] path_list ([string,...]) List of file that start with env.get_build_system_base_name() in the running worktree (Parse one time ==> faster)
##
def import_path(path_list):
	global __system_list
	global_base = env.get_build_system_base_name()
	debug.debug("SYSTEM: Init with Files list:")
	for elem in path_list:
		sys.path.append(os.path.dirname(elem))
		# Get file name:
		filename = os.path.basename(elem)
		# Remove .py at the end:
		filename = filename[:-3]
		# Remove global base name:
		filename = filename[len(global_base):]
		# Check if it start with the local patern:
		if filename[:len(__start_system_name)] != __start_system_name:
			debug.extreme_verbose("SYSTEM:     NOT-Integrate: '" + filename + "' from '" + elem + "' ==> rejected")
			continue
		# Remove local patern
		system_name = filename[len(__start_system_name):]
		system_type, system_name = system_name.split('_')
		debug.verbose("SYSTEM:     Integrate: '" + system_type + "':'" + system_name + "' from '" + elem + "'")
		if system_type in __system_list:
			__system_list[system_type].append({"name":system_name,
			                                   "path":elem,
			                                   "system":None,
			                                   "loaded":False,
			                                   "exist":False,
			                                   "module":None})
		else:
			__system_list[system_type] = [{"name":system_name,
			                               "path":elem,
			                               "system":None,
			                               "loaded":False,
			                               "exist":False,
			                               "module":None}]
	debug.verbose("New list system: ")
	for elem in __system_list:
		debug.verbose("    " + str(elem))
		for val in __system_list[elem]:
			debug.verbose("        " + str(val["name"]))

##
## @brief Display all the system binary that can be used
##
def display():
	global __system_list
	for elementName in __system_list:
		debug.info("SYSTEM:     Integrate system: '" + elementName +"'")
		for data in __system_list[elementName]:
			debug.info("SYSTEM:    '" + data["name"] +"' in " + data["path"])

##
## @brief Check if a system Module is availlable for a specific target
## @param[in] lib_name (string) Name of the Library
## @param[in] list_target_name ([string,...]) list of name of the target (ordered by request order)
## @param[in] target (handle) Handle on the @ref Target build engine
## @return (bool) find the system lib or not
##
def exist(lib_name, list_target_name, target) :
	global __system_list
	debug.verbose("exist= " + lib_name + " in " + str(list_target_name))
	find_target = False
	for target_name in list_target_name:
		if target_name in __system_list:
			find_target = True
	if find_target == False:
		return False
	for target_name in reversed(list_target_name):
		if target_name not in __system_list:
			continue
		for data in __system_list[target_name]:
			if data["name"] == lib_name:
				# we find it in the List ==> need to check if it is present in the system :
				if data["loaded"] == False:
					debug.verbose("add to path: '" + os.path.dirname(data["path"]) + "'")
					sys.path.append(os.path.dirname(data["path"]))
					debug.verbose("import system : '" + data["name"] + "'")
					the_system = __import__(env.get_build_system_base_name() + __start_system_name + target_name + "_" + data["name"])
					#create the system module
					debug.verbose("SYSTEM: request: " + str(data["name"]))
					if "System" in dir(the_system):
						data["system"] = the_system.System(target)
						data["exist"] = data["system"].get_valid()
						"""
						if data["exist"] == False:
							debug.warning("Can not Import: '" + data["name"] + "' ==> disabled")
						"""
					else:
						debug.warning("Not find: '" + data["name"] + "' ==> get exception")
				return data["exist"]
	return False

##
## @brief Load a system Module for a specific target
## @param[in] target (handle) Handle on the @ref Target build engine
## @param[in] lib_name (string) Name of the Library
## @param[in] list_target_name ([string,...]) list of name of the target (ordered by request order)
## @return None
##
def load(target, lib_name, list_target_name):
	global __system_list
	find_target = False
	for target_name in list_target_name:
		if target_name in __system_list:
			find_target = True
	if find_target == False:
		debug.error("you must call this function after checking of the system exist() !1!")
		return
	for target_name in reversed(list_target_name):
		if target_name not in __system_list:
			continue
		for data in __system_list[target_name]:
			if data["name"] == lib_name:
				if data["exist"] == False:
					debug.error("you must call this function after checking of the system exist() !2!")
				if data["module"] == None:
					# create a module from the system interface...
					data["module"] = create_module_from_system(target, data)
					data["loaded"] = True
				target.add_module(data["module"])
				return




#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

##
## ASM builder
##
from lutin import multiprocess
from lutin import tools
from lutin import depend
from lutin import env

##
## Initialize the builder, if needed ... to get dependency between builder (for example)
##
def init():
	pass

##
## Get the current builder type.
## Return the type of builder
##
def get_type():
	return "compiler"

##
## @brief Get builder input file type
## @return List of extention supported
##
def get_input_type():
	return ["s", "S"]

##
## @brief Get builder output file type
## @return List of extention supported
##
def get_output_type():
	return ["o"]

##
## @brief Get builder support multi-threading or not
## @return True Multithreading supported
## @return False Multithreading NOT supported
##
def get_support_multithreading():
	return True

##
## @brief Commands for running gcc to compile a C file in object file.
##
def compile(file, binary, target, depancy, flags, path, name, basic_path, module_src):
	file_src = target.get_full_name_source(basic_path, file)
	file_cmd = target.get_full_name_cmd(name, basic_path, file)
	file_dst = target.get_full_name_destination(name, basic_path, file, get_output_type())
	file_depend = target.get_full_dependency(name, basic_path, file)
	file_warning = target.get_full_name_warning(name, basic_path, file)
	
	# set ccache interface:
	compilator_ccache = ""
	if env.get_ccache() == True:
		compilator_ccache = "ccache"
	# create the command line befor requesting start:
	cmd = [
		compilator_ccache,
		target.cc,
		"-o", file_dst,
		target.arch,
		target.sysroot]
	for view in ["export", "local"]:
		try:
			cmd.append(tools.add_prefix("-I", path[view]["c"]))
		except:
			pass
	try:
		cmd.append(tools.add_prefix("-I", depancy.path["c"]))
	except:
		pass
	cmd.append(target.global_include_cc)
	try:
		cmd.append(target.global_flags["c"])
	except:
		pass
	try:
		cmd.append(depancy.flags["c"])
	except:
		pass
	for view in ["local", "export"]:
		try:
			cmd.append(flags[view]["c"])
		except:
			pass
	cmd.append("-c")
	cmd.append("-MMD")
	cmd.append("-MP")
	cmd.append(file_src)
	# Create cmd line
	cmdLine=tools.list_to_str(cmd)
	# check the dependency for this file :
	if depend.need_re_build(file_dst, file_src, file_depend, file_cmd, cmdLine) == False:
		return {"action":"add", "file":file_dst}
	tools.create_directory_of_file(file_dst)
	comment = ["s", name, "<==", file]
	# process element
	multiprocess.run_in_pool(cmdLine, comment, file_cmd, store_output_file=file_warning)
	return {"action":"add", "file":file_dst}



#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

from realog import debug
from lutin import target
from lutin import tools
from lutin import env
import os
import stat
import re
from lutin import host
from lutin import depend
from lutin import multiprocess

class Target(target.Target):
	def __init__(self, config, sub_name=[]):
		#processor type selection (auto/arm/ppc/x86)
		if config["arch"] == "auto":
			config["arch"] = "x86"
		#bus size selection (auto/32/64)
		if config["bus-size"] == "auto":
			config["bus-size"] = str(host.BUS_SIZE)
		target.Target.__init__(self, ["Linux"] + sub_name, config, "")
		if self.config["bus-size"] == "64":
			# 64 bits
			if host.BUS_SIZE != 64:
				self.add_flag("c", "-m64")
		else:
			# 32 bits
			if host.BUS_SIZE != 32:
				self.add_flag("c", "-m32")
		
		self.add_flag("c", "-fPIC")
		
		self.pkg_path_data = "share"
		self.pkg_path_bin = "bin"
		self.pkg_path_lib = "lib"
		self.pkg_path_license = "license"
		# disable sysroot when generate build in isolated mode
		if env.get_isolate_system() == True:
			self.sysroot = "--sysroot=/aDirectoryThatDoesNotExist/"
		
		self.add_flag("link-lib", [
		    "dl"
		    ])
		self.add_flag("link", [
		    "-rdynamic"
		    ])
	
	"""
	.local/application
	    *--> applName -> applName.app/bin/applName
	    *--> applName.app
	        *--> appl_description.txt
	        *--> appl_name.txt
	        *--> changelog.txt
	        *--> copyright.txt
	        *--> readme.txt
	        *--> version.txt
	        *--> website.txt
	        *--> icon.png
	        *--> bin
	        *    *--> applName
	        *--> doc
	        *    *--> applName
	        *--> lib
	        *    *--> XX.so
	        *    *--> YY.so
	        *--> license
	        *    *--> applName.txt
	        *    *--> libXX.txt
	        *    *--> libYY.txt
	        *--> man
	        *--> share
	        *    *--> applName
	        *    *--> XX
	        *    *--> YY
	        *--> sources
	"""
	def make_package_binary(self, pkg_name, pkg_properties, base_pkg_path, heritage_list, static):
		debug.debug("------------------------------------------------------------------------")
		debug.debug("-- Generate generic '" + pkg_name + "' v" + tools.version_to_string(pkg_properties["VERSION"]))
		debug.debug("------------------------------------------------------------------------")
		#output path
		target_outpath = os.path.join(self.get_staging_path(pkg_name), pkg_name + ".app")
		tools.create_directory_of_file(target_outpath)
		
		## Create share datas:
		ret_share = self.make_package_binary_data(target_outpath, pkg_name, base_pkg_path, heritage_list, static)
		
		## copy binary files:
		ret_bin = self.make_package_binary_bin(target_outpath, pkg_name, base_pkg_path, heritage_list, static)
		
		## Create libraries:
		ret_lib = self.make_package_binary_lib(target_outpath, pkg_name, base_pkg_path, heritage_list, static)
		
		## Create generic files:
		ret_file = self.make_package_generic_files(target_outpath, pkg_properties, pkg_name, base_pkg_path, heritage_list, static)
		
		## end of the package generation
		build_package_path_done = os.path.join(self.get_build_path(pkg_name), "generatePackageDone.txt")
		#Check date between the current file "list of action to generate package and the end of package generation
		need_generate_package = depend.need_re_package(build_package_path_done, [__file__], True)
		
		## create the package:
		if    ret_share \
		   or ret_bin \
		   or ret_lib \
		   or ret_file \
		   or need_generate_package:
			debug.debug("package : " + os.path.join(self.get_staging_path(pkg_name), pkg_name + ".app.pkg"))
			os.system("cd " + self.get_staging_path(pkg_name) + " ; tar -czf " + pkg_name + ".app.tar.gz " + pkg_name + ".app")
			#multiprocess.run_command("cd " + self.get_staging_path(pkg_name) + " ; tar -czf " + pkg_name + ".app.tar.gz " + pkg_name + ".app")
			tools.create_directory_of_file(self.get_final_path())
			tools.copy_file(os.path.join(self.get_staging_path(pkg_name), pkg_name + ".app.tar.gz"), os.path.join(self.get_final_path(), pkg_name + ".app.gpkg"))
			# package is done corectly ...
			tools.file_write_data(build_package_path_done, "done...")
	
	def install_package(self, pkg_name):
		debug.debug("------------------------------------------------------------------------")
		debug.info("-- Install package '" + pkg_name + "'")
		debug.debug("------------------------------------------------------------------------")
		# this is temporary ... Will call:
		if False:
			os.system("lutin-pkg -i " + os.path.join(self.get_final_path(), + pkg_name + ".app.gpkg"))
		else:
			#Copy directly from staging path:
			appl_path = os.path.join(self.get_staging_path(pkg_name), pkg_name + ".app")
			target_path = os.path.join(os.path.expanduser("~"), ".local", "application", pkg_name + ".app")
			target_bin_path = os.path.join(os.path.expanduser("~"), ".local", "application", pkg_name + ".app", "bin", pkg_name)
			target_bin_link = os.path.join(os.path.expanduser("~"), ".local", "application", pkg_name)
			# remove output path:
			tools.remove_path_and_sub_path(target_path)
			# remove executable link version:
			tools.remove_file(target_bin_link)
			# copy result:
			tools.copy_anything(appl_path, target_path, recursive=True)
			# create synbolic link:
			debug.info("kkk " + "ln -s " + target_bin_path + " " + target_bin_link)
			os.symlink(target_bin_path, target_bin_link)
	
	def un_install_package(self, pkg_name):
		debug.debug("------------------------------------------------------------------------")
		debug.info("-- Un-Install package '" + pkg_name + "'")
		debug.debug("------------------------------------------------------------------------")
		# this is temporary ... Will call:
		if False:
			os.system("lutin-pkg -r " + pkg_name)
		else:
			#Copy directly from staging path:
			target_path = os.path.join(os.path.expanduser("~"), ".local", "application", pkg_name + ".app")
			target_bin_link = os.path.join(os.path.expanduser("~"), ".local", "application", pkg_name)
			# remove output path:
			tools.remove_path_and_sub_path(target_path)
			# remove executable link version:
			tools.remove_file(target_bin_link)
	
	def run(self, pkg_name, option_list, binary_name = None):
		if binary_name == None:
			binary_name = pkg_name;
		debug.debug("------------------------------------------------------------------------")
		debug.info("-- Run package '" + pkg_name + "' executable: '" + binary_name + "' + option: " + str(option_list))
		debug.debug("------------------------------------------------------------------------")
		appl_path = os.path.join(self.get_staging_path(pkg_name), pkg_name + ".app", "bin", binary_name)
		cmd = appl_path + " "
		for elem in option_list:
			cmd += elem + " "
		multiprocess.run_command_no_lock_out(cmd)
		debug.debug("------------------------------------------------------------------------")
		debug.info("-- Run package '" + pkg_name + "' Finished")
		debug.debug("------------------------------------------------------------------------")





#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##
import os
# Local import
from realog import debug
from . import env

def _create_directory_of_file(file):
	path = os.path.dirname(file)
	try:
		os.stat(path)
	except:
		os.makedirs(path)

def _file_write_data(path, data):
	file = open(path, "w")
	file.write(data)
	file.close()

def _file_size(path):
	if not os.path.isfile(path):
		return 0
	statinfo = os.stat(path)
	return statinfo.st_size

def _file_read_data(path, binary=False):
	if not os.path.isfile(path):
		return ""
	if binary == True:
		file = open(path, "rb")
	else:
		file = open(path, "r")
	data_file = file.read()
	file.close()
	return data_file

##
## @brief Creata a dependency file with a list of files
## @param[in] depend_file (string) filename to store a dependency data
## @param[in] list_files ([string,...]) List of file that depend the current element
##
def create_dependency_file(depend_file, list_files):
	data = ""
	for elem in list_files:
		data += elem + "\n"
	_create_directory_of_file(depend_file)
	_file_write_data(depend_file, data)

##
## @brief Check if all dependency of a file and dependency file is correct or not
## @param[in] dst (string) File that will be generated
## @param[in] src (string) Source File needed to generate the 'dst'
## @param[in] depend_file (string) Dependency file that contain all file that the src depending
## @param[in] file_cmd (string) Filename of where is store the cmdline to generate the 'dst'
## @param[in] cmd_line (string) Command line that will be use to generate the 'dst'
## @param[in] force_identical (string) for copy file this check if the src and dst are identical
## @return (bool) True: something change ==> need to rebuild, False otherwise
##
def need_re_build(dst, src, depend_file=None, file_cmd="", cmd_line="", force_identical=False):
	debug.extreme_verbose("Request check of dependency of :")
	debug.extreme_verbose("		dst='" + str(dst) + "'")
	debug.extreme_verbose("		src='" + str(src) + "'")
	debug.extreme_verbose("		dept='" + str(depend_file) + "'")
	debug.extreme_verbose("		cmd='" + str(file_cmd) + "'")
	debug.extreme_verbose("		force_identical='" + str(force_identical) + "'")
	# if force mode selected ==> just force rebuild ...
	if env.get_force_mode():
		debug.extreme_verbose("			==> must rebuild (force mode)")
		return True
	
	# check if the destination existed:
	if     dst != "" \
	   and dst != None \
	   and os.path.exists(dst) == False:
		debug.extreme_verbose("			==> must rebuild (dst does not exist)")
		return True
	if     src != "" \
	   and src != None \
	   and os.path.exists(src) == False:
		debug.warning("			==> unexistant file :'" + src + "'")
		return True
	# Check the basic date if the 2 files
	if     dst != "" \
	   and dst != None \
	   and src != "" \
	   and src != None \
	   and os.path.getmtime(src) > os.path.getmtime(dst):
		debug.extreme_verbose("			==> must rebuild (source time greater)")
		return True
	
	if     depend_file != "" \
	   and depend_file != None \
	   and os.path.exists(depend_file) == False:
		debug.extreme_verbose("			==> must rebuild (no depending file)")
		return True
	
	if     file_cmd != "" \
	   and file_cmd != None:
		if os.path.exists(file_cmd) == False:
			debug.extreme_verbose("			==> must rebuild (no commandLine file)")
			return True
		# check if the 2 cmd_line are similar :
		file2 = open(file_cmd, "r")
		first_and_unique_line = file2.read()
		if first_and_unique_line != cmd_line:
			debug.extreme_verbose("			==> must rebuild (cmd_lines are not identical)")
			debug.extreme_verbose("				==> '" + cmd_line + "'")
			debug.extreme_verbose("				==> '" + first_and_unique_line + "'")
			file2.close()
			return True
		# the cmdfile is correct ...
		file2.close()
	
	if     depend_file != "" \
	   and depend_file != None:
		debug.extreme_verbose("			start parsing dependency file : '" + depend_file + "'")
		file = open(depend_file, "r")
		for cur_line in file.readlines():
			# normal file : end with : ": \\n"
			cur_line = cur_line[:len(cur_line)-1]
			# removing last \ ...
			if cur_line[len(cur_line)-1:] == '\\' :
				cur_line = cur_line[:len(cur_line)-1]
			# remove white space : 
			#debug.verbose("				Line (read) : '" + cur_line + "'");
			cur_line = cur_line.strip()
			#debug.verbose("				Line (strip) : '" + cur_line + "'");
			
			test_file=""
			if cur_line[len(cur_line)-1:] == ':':
				debug.extreme_verbose("				Line (no check (already done) : '" + cur_line + "'");
			elif    len(cur_line) == 0 \
			     or cur_line == '\\':
				debug.extreme_verbose("				Line (Not parsed) : '" + cur_line + "'");
			else:
				test_file = cur_line
				debug.extreme_verbose("				Line (might check) : '" + test_file + "'");
			# really check files:
			if test_file != "":
				debug.extreme_verbose("					==> test");
				if False==os.path.exists(test_file):
					debug.extreme_verbose("			==> must rebuild (a dependency file does not exist)")
					file.close()
					return True
				if os.path.getmtime(test_file) > os.path.getmtime(dst):
					debug.extreme_verbose("			==> must rebuild (a dependency file time is newer)")
					file.close()
					return True
		# close the current file :
		file.close()
	# check the 2 files are identical:
	if force_identical == True:
		# check if the 2 cmd_line are similar:
		size_src = _file_size(src)
		size_dst = _file_size(dst)
		if size_src != size_dst:
			debug.extreme_verbose("			Force Rewrite not the same size     size_src=" + str(size_src) + " != size_dest=" + str(size_dst))
			return True
		data_src = _file_read_data(src, binary=True)
		data_dst = _file_read_data(dst, binary=True)
		if data_src != data_dst:
			debug.extreme_verbose("			Force Rewrite not the same data")
			return True
	
	debug.extreme_verbose("			==> Not rebuild (all dependency is OK)")
	return False

##
## @brief 
## @param[in] dst (string) File that will be generated
## @param[in] src_list ([string,...]) Source file list needed to generate the 'dst'
## @param[in] must_have_src (bool) All sources must be present
## @param[in] file_cmd (string) Filename of where is store the cmdline to generate the 'dst'
## @param[in] cmd_line (string) Command line that will be use to generate the 'dst'
## @return (bool) True: Need to regenerate the package, False otherwise
##
def need_re_package(dst, src_list, must_have_src, file_cmd="", cmd_line=""):
	debug.extreme_verbose("Request check of dependency of :")
	debug.extreme_verbose("		dst='" + str(dst) + "'")
	compleate_list = []
	debug.extreme_verbose("		src:")
	if type(src_list) == str:
		compleate_list.append(src_list)
		debug.extreme_verbose("			'" + src_list + "'")
	elif type(src_list) == list:
		for src in src_list:
			compleate_list.append(src)
			debug.extreme_verbose("			'" + str(src) + "'")
	elif type(src_list) == dict:
		for key in src_list:
			debug.extreme_verbose("			'" + str(key) + "'")
			for src in src_list[key]:
				compleate_list.append(src)
				debug.extreme_verbose("				'" + str(src) + "'")
	
	if     must_have_src == False \
	   and len(compleate_list) == 0:
		return False
	
	# if force mode selected ==> just force rebuild ...
	if env.get_force_mode():
		debug.extreme_verbose("			==> must re-package (force mode)")
		return True
	
	# check if the destination existed:
	if os.path.exists(dst) == False:
		debug.extreme_verbose("			==> must re-package (dst does not exist)")
		return True
	# chek the basic date if the 2 files
	if len(compleate_list) == 0:
		debug.extreme_verbose("			==> must re-package (no source ???)")
		return True
	for src in compleate_list:
		if os.path.getmtime(src) > os.path.getmtime(dst):
			debug.extreme_verbose("			==> must re-package (source time greater) : '" + src + "'")
			return True
	
	if ""!=file_cmd:
		if False==os.path.exists(file_cmd):
			debug.extreme_verbose("			==> must rebuild (no commandLine file)")
			return True
		# check if the 2 cmd_line are similar :
		file2 = open(file_cmd, "r")
		first_and_unique_line = file2.read()
		if first_and_unique_line != cmd_line:
			debug.extreme_verbose("			==> must rebuild (cmd_lines are not identical)")
			debug.extreme_verbose("				==> '" + cmd_line + "'")
			debug.extreme_verbose("				==> '" + first_and_unique_line + "'")
			file2.close()
			return True
		# the cmdfile is correct ...
		file2.close()
	
	debug.extreme_verbose("			==> Not re-package (all dependency is OK)")
	return False






#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##
import platform
import os
# Local import
from realog import debug
from . import tools
from . import multiprocess
from . import depend

enable_resize_image = True
try:
	if platform.system() == "Darwin":
		import CoreGraphics
	else:
		from PIL import Image
except:
	enable_resize_image = False
	debug.warning("Missing python tools : CoreGraphics (MacOs) or PIL") 

##
## @brief Generate the upper pow 2 of the size in parameter
## @param[in] size (int) Size that might heve the upper pow(2)
## @return the new size in pow(2)
##
def get_pow_2_multiple(size):
	base = 2
	while size>base:
		base = base * 2
	return base

##
## @brief Resize an image
## @param[in] src_file (string) Path of the source image file
## @param[in] dest_file (string) Path out the destination image file
## @param[in] x (int) Width of the image
## @param[in] y (int) Height of the image
## @param[in] cmd_file (string) Filename of where is store the cmdline to generate the 'dst'
##
# TODO : 3 things to do :
#        check if force requested
#        check if time change
#        check if command line change
def resize(src_file, dest_file, x, y, cmd_file=None):
	if os.path.exists(src_file) == False:
		debug.error("Request a resize an image that does not existed : '" + src_file + "'")
		return
	if enable_resize_image == False:
		debug.warning("Can not resize image missing pillow or CoreGraphics : '" + src_file + "' (just copy)")
		tools.copy_file(src_file, dest_file)
		return
	cmd_line = "resize Image : " + src_file + " ==> " + dest_file + " newSize=(" + str(x) + "x" + str(y) + ")"
	if depend.need_re_build(dest_file, src_file, file_cmd=cmd_file , cmd_line=cmd_line) == False:
		return
	# add cmdLine ...
	x = get_pow_2_multiple(x)
	extension = dest_file[dest_file.rfind('.'):]
	if platform.system() == "Darwin":
		source_image = CoreGraphics.CGImageImport(CoreGraphics.CGDataProviderCreateWithFilename(src_file))
		source_width = source_image.getWidth()
		source_height = source_image.getHeight()
		if source_width <= x:
			# for small image just copy:
			tools.copy_file(src_file, dest_file)
		else:
			if y <= 0:
				# keep ratio :
				y = int(float(x) * float(source_height) / float(source_width))
			y = get_pow_2_multiple(y)
			debug.print_element("resize Image (" + str(x) + "x" + str(y) + ")", src_file, "==>", dest_file)
			debug.debug("Resize image: " + src_file + " size=(" + str(source_width) + "x" + str(source_height) + ") -> (" + str(x) + "x" + str(y) + ")")
			source_image_rect = CoreGraphics.CGRectMake(0, 0, source_width, source_height)
			new_image = source_image.createWithImageInRect(source_image_rect)
			colors_space = CoreGraphics.CGColorSpaceCreateDeviceRGB()
			colors = CoreGraphics.CGFloatArray(5)
			context = CoreGraphics.CGBitmapContextCreateWithColor(x, y, colors_space, colors)
			context.setInterpolationQuality(CoreGraphics.kCGInterpolationHigh)
			new_image_rect = CoreGraphics.CGRectMake(0, 0, x, y)
			context.drawImage(new_image_rect, new_image)
			tools.create_directory_of_file(dest_file)
			if extension == ".jpeg":
				context.writeToFile(dest_file, CoreGraphics.kCGImageFormatJPEG)
			elif extension == ".png":
				context.writeToFile(dest_file, CoreGraphics.kCGImageFormatPNG)
			else:
				debug.error(" can not manage extention ... : " + dest_file)
	else:
		# open an image file (.bmp,.jpg,.png,.gif) you have in the working path
		im1 = Image.open(src_file)
		if im1.size[0] <= x:
			# for small image just copy:
			tools.copy_file(src_file, dest_file)
		else:
			if y <= 0:
				# keep ratio :
				y = int(float(x) * float(im1.size[1]) / float(im1.size[0]))
			y = get_pow_2_multiple(y)
			debug.print_element("resize Image (" + str(x) + "x" + str(y) + ")", src_file, "==>", dest_file)
			# use one of these filter options to resize the image
			tmpImage = im1.resize((x, y), Image.ANTIALIAS)
			tools.create_directory_of_file(dest_file)
			tmpImage.save(dest_file)
	tools.store_command(cmd_line, cmd_file)



#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

from realog import debug
from lutin import system
from lutin import tools
from lutin import env
import os

class System(system.System):
	def __init__(self, target):
		system.System.__init__(self)
		# create some HELP:
		self.set_help("pthread : Generic multithreading system\n Can be install with the package:\n    - pthread-dev")
		# check if the library exist:
		"""
		if     not os.path.isfile("/usr/include/pthread.h"):
			# we did not find the library reqiested (just return) (automaticly set at false)
			return;
		"""
		self.set_valid(True)
		# todo : create a searcher of the presence of the library:
		#self.add_flag("link-lib", "pthread")
		self.add_depend([
		    'c'
		    ])





#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

from realog import debug
from lutin import target
from lutin import tools
from lutin import image
from lutin import multiprocess
from lutin import host
from lutin import depend
import os
import sys

class Target(target.Target):
	def __init__(self, config, sub_name=[]):
		#processor type selection (auto/arm/ppc/x86)
		if config["arch"] == "auto":
			config["arch"] = "arm"
		#bus size selection (auto/32/64)
		if config["bus-size"] == "auto":
			config["bus-size"] = "32"
		self.type_arch = ""
		target.Target.__init__(self, ["Android"] + sub_name, config, self.type_arch)
		
		if config["bus-size"] == "32":
			self.type_arch="armv7"
		else:
			self.type_arch="arm64"
		
		self.path_ndk = os.getenv('PROJECT_NDK', "AUTO")
		self.path_sdk = os.getenv('PROJECT_SDK', "AUTO")
		# auto search NDK
		if self.path_ndk == "AUTO":
			for path in os.listdir("."):
				if os.path.isdir(path)==True:
					if path=="android":
						self.path_ndk = path + "/ndk"
			if self.path_ndk == "AUTO":
				self.path_ndk = tools.get_run_path() + "/../android/ndk"
		# auto search SDK
		if self.path_sdk == "AUTO":
			for path in os.listdir("."):
				if os.path.isdir(path)==True:
					if path=="android":
						self.path_sdk = path + "/sdk"
			if self.path_sdk == "AUTO":
				self.path_sdk = tools.get_run_path() + "/../android/sdk"
		
		if not os.path.isdir(self.path_ndk):
			debug.error("NDK path not set !!! set env : PROJECT_NDK on the NDK path")
		if not os.path.isdir(self.path_sdk):
			debug.error("SDK path not set !!! set env : PROJECT_SDK on the SDK path")
		
		
		tmpOsVal = "64"
		gccVersion = "4.9"
		# TODO : Remove this or set it better ...
		self.compilator_version = gccVersion
		if host.BUS_SIZE==64:
			tmpOsVal = "_64"
		if self.config["compilator"] == "clang":
			self.set_cross_base(self.path_ndk + "/toolchains/llvm-3.6/prebuilt/linux-x86" + tmpOsVal + "/bin/")
			# Patch for LLVM AR tool
			self.ar = self.cross + "llvm-ar"
		else:
			basepathArm = self.path_ndk + "/toolchains/arm-linux-androideabi-" + gccVersion + "/prebuilt/linux-x86" + tmpOsVal + "/bin/"
			basepathMips = self.path_ndk + "/toolchains/mipsel-linux-android-" + gccVersion + "/prebuilt/linux-x86" + tmpOsVal + "/bin/"
			basepathX86 = self.path_ndk + "/toolchains/x86-" + gccVersion + "/prebuilt/linux-x86" + tmpOsVal + "/bin/"
			self.set_cross_base(basepathArm + "arm-linux-androideabi-")
			if not os.path.isdir(basepathArm):
				debug.error("Gcc Arm path does not exist !!!")
			if not os.path.isdir(basepathMips):
				debug.info("Gcc Mips path does not exist !!!")
			if not os.path.isdir(basepathX86):
				debug.info("Gcc x86 path does not exist !!!")
		
		# TODO : Set it back in the package only ...
		#self.path_bin="mustNotCreateBinary"
		#self.path_lib="data/lib/armeabi"
		#self.path_data="data/assets"
		#self.path_doc="doc"
		#self.suffix_package='.pkg'
		self.pkg_path_data = "data/assets"
		self.pkg_path_bin = "mustNotCreateBinary"
		self.pkg_path_lib = "data/lib/armeabi"
		self.pkg_path_license = "license"
		
		# If the env variable is not define, find the newest version of the BOARD_ID (Note: 0: autofind)
		self.board_id = int(os.getenv('PROJECT_NDK_BOARD_ID', "0"))
		if self.board_id != 0:
			# check if element existed :
			if not os.path.isdir(self.path_sdk +"/platforms/android-" + str(self.board_id)):
				debug.error("Specify PROJECT_NDK_BOARD_ID env variable and the BOARD_ID does not exit ... : " + str(self.board_id) + "==> auto-search")
				self.board_id = 0
		if self.board_id == 0:
			debug.debug("Auto-search BOARD-ID")
			for iii in reversed(range(0, 50)):
				debug.debug("try: " + os.path.join(self.path_sdk, "platforms", "android-" + str(iii)))
				if os.path.isdir(os.path.join(self.path_sdk, "platforms", "android-" + str(iii))):
					debug.debug("Find BOARD-ID : " + str(iii))
					self.board_id = iii
					break;
		if self.board_id == 0:
			debug.error("Can not find BOARD-ID ==> update your android SDK")
		
		self.add_flag("c", "-D__ANDROID_BOARD_ID__=" + str(self.board_id))
		if self.type_arch == "armv5" or self.type_arch == "armv7":
			self.global_include_cc.append("-I" + os.path.join(self.path_ndk, "platforms", "android-" + str(self.board_id), "arch-arm", "usr", "include"))
		elif self.type_arch == "mips":
			self.global_include_cc.append("-I" + os.path.join(self.path_ndk, "platforms", "android-" + str(self.board_id), "arch-mips", "usr", "include"))
		elif self.type_arch == "x86":
			self.global_include_cc.append("-I" + os.path.join(self.path_ndk, "platforms", "android-" + str(self.board_id), "arch-x86", "usr", "include"))
		
		self.global_include_cc.append("-I" + os.path.join(self.path_ndk, "sources", "android", "support", "include"))
		
		if self.config["compilator"] == "clang":
			self.global_include_cc.append("-gcc-toolchain " + os.path.join(self.path_ndk, "sources", "android", "support", "include"))
			if self.type_arch == "armv5":
				pass
			elif self.type_arch == "armv7":
				# The only one tested ... ==> but we have link error ...
				self.add_flag("c", [
				    "-target armv7-none-linux-androideabi",
				    "-march=armv7-a",
				    "-mfpu=vfpv3-d16",
				    "-mhard-float"
				    ])
				self.add_flag("link", [
				    "-target armv7-none-linux-androideabi",
				    "-Wl,--fix-cortex-a8",
				    "-Wl,--no-warn-mismatch",
				    "-lm_hard"
				    ])
			elif self.type_arch == "mips":
				pass
			elif self.type_arch == "x86":
				pass
		else:
			if self.type_arch == "armv5":
				pass
			elif self.type_arch == "armv7":
				pass
			elif self.type_arch == "mips":
				pass
			elif self.type_arch == "x86":
				pass
		
		self.sysroot = "--sysroot=" + os.path.join(self.path_ndk, "platforms", "android-" + str(self.board_id), "arch-arm")
		
		if True:
			self._mode_arm = "thumb"
		else:
			self._mode_arm = "arm"
		
		
		self.add_flag("c", [
		    "-D__ARM_ARCH_5__",
		    "-D__ARM_ARCH_5T__",
		    "-D__ARM_ARCH_5E__",
		    "-D__ARM_ARCH_5TE__"
		    ])
		if self.config["compilator"] != "clang":
			if self.type_arch == "armv5":
				# -----------------------
				# -- arm V5 :
				# -----------------------
				self.add_flag("c", [
				    "-march=armv5te",
				    "-msoft-float"
				    ])
			else:
				# -----------------------
				# -- arm V7 (Neon) :
				# -----------------------
				self.add_flag("c", [
				    "-m"+self._mode_arm,
				    "-mfpu=neon",
				    "-march=armv7-a",
				    "-mtune=cortex-a8",
				    "-mfloat-abi=softfp",
				    "-mvectorize-with-neon-quad",
				    "-D__ARM_ARCH_7__",
				    "-D__ARM_NEON__"
				    ])
				self.add_flag("link", [
				    "-mfpu=neon",
				    "-mfloat-abi=softfp",
				    "-Wl,--fix-cortex-a8",
				    ])
		"""
		self.add_flag("link-lib", [
		    "dl"
		    ])
		self.add_flag("link", [
		    "-rdynamic"
		    ])
		"""
		# the -mthumb must be set for all the android produc, some ot the not work coretly without this one ... (all android code is generated with this flags)
		#self.add_flag("c", "-mthumb")
		# -----------------------
		# -- Common flags :
		# -----------------------
		self.add_flag("c", "-fpic")
		if self.config["compilator"] != "clang":
			self.add_flag("c", [
			    "-ffunction-sections",
			    "-funwind-tables",
			    "-fstack-protector",
			    "-Wno-psabi",
			    #"-mtune=xscale",
			    "-fomit-frame-pointer",
			    "-fno-strict-aliasing"
			    ])
		self.add_flag("c++", [
		    "-frtti",
		    "-fexceptions",
		    "-Wa,--noexecstack"
		    ])
	
	##
	## @brief Get the arm mode that is build
	## @return "thumb" for thumb mode and "arm for normal mode
	##
	def get_arm_mode(self):
		return self._mode_arm
	
	def convert_name_application(self, pkg_name):
		value = pkg_name.lower()
		value = value.replace(' ', '')
		value = value.replace('-', '')
		value = value.replace('_', '')
		return value
	
	"""
	def get_staging_path_data(self, binary_name):
		return self.get_staging_path(binary_name) + self.path_data
	"""
	
	def make_package_binary(self, pkg_name, pkg_properties, base_pkg_path, heritage_list, static):
		debug.debug("------------------------------------------------------------------------")
		debug.info("Generate package '" + pkg_name + "' v" + tools.version_to_string(pkg_properties["VERSION"]))
		debug.debug("------------------------------------------------------------------------")
		#output path
		target_outpath = self.get_staging_path(pkg_name)
		tools.create_directory_of_file(target_outpath)
		
		## Create share datas:
		self.make_package_binary_data(target_outpath, pkg_name, base_pkg_path, heritage_list, static)
		
		## copy binary files
		# in Android Package we have no binary element, only shared object ... (and java start file)
		
		## Create libraries (special case of Android...)
		copy_list={}
		target_outpath_lib = os.path.join(target_outpath, self.pkg_path_lib)
		tools.create_directory_of_file(target_outpath_lib)
		# copy application lib: (needed to lunch ...)
		file_src = self.get_build_file_dynamic(pkg_name)
		if os.path.isfile(file_src):
			debug.debug("      need copy: " + file_src + " to " + target_outpath_lib)
			tools.copy_file(file_src,
			                os.path.join(target_outpath_lib, os.path.basename(file_src)),
			                in_list=copy_list)
		# copy other if needed:
		if static == False:
			#copy all shared libsh...
			debug.verbose("libs for " + str(pkg_name) + ":")
			for heritage in heritage_list.list_heritage:
				debug.debug("sub elements: " + str(heritage.name))
				file_src = self.get_build_file_dynamic(heritage.name)
				debug.verbose("      has directory: " + file_src)
				if os.path.isfile(file_src):
					debug.debug("      need copy: " + file_src + " to " + target_outpath_lib)
					#copy all data:
					# TODO : We can have a problem when writing over library files ...
					tools.copy_file(file_src,
					                os.path.join(target_outpath_lib, os.path.basename(file_src)),
					                in_list=copy_list)
		#real copy files
		tools.copy_list(copy_list)
		if self.pkg_path_lib != "":
			# remove unneded files (NOT folder ...)
			tools.clean_directory(target_outpath_lib, copy_list)
		
		## Create generic files:
		self.make_package_generic_files(target_outpath, pkg_properties, pkg_name, base_pkg_path, heritage_list, static)
		
		## create specific android project (local)
		pkg_name_application_name = pkg_name
		if self.config["mode"] == "debug":
			pkg_name_application_name += "debug"
		#debug.info("ploppppp: " + str(pkg_properties))
		# FINAL_path_JAVA_PROJECT
		self.path_java_project = os.path.join(target_outpath,
		                                      "src")
		if pkg_properties["COMPAGNY_TYPE"] != "":
			self.path_java_project = os.path.join(self.path_java_project,
			                                      pkg_properties["COMPAGNY_TYPE"])
		if pkg_properties["COMPAGNY_NAME2"] != "":
			self.path_java_project = os.path.join(self.path_java_project,
			                                      pkg_properties["COMPAGNY_NAME2"])
		self.path_java_project = os.path.join(self.path_java_project,
		                                      pkg_name_application_name)
		#FINAL_FILE_ABSTRACTION
		self.file_final_abstraction = os.path.join(self.path_java_project, pkg_name_application_name + ".java")
		
		compleatePackageName = ""
		if pkg_properties["COMPAGNY_TYPE"] != "":
			compleatePackageName += pkg_properties["COMPAGNY_TYPE"] + "."
		if pkg_properties["COMPAGNY_NAME2"] != "":
			compleatePackageName += pkg_properties["COMPAGNY_NAME2"] + "."
		compleatePackageName += pkg_name_application_name
		
		if "ADMOD_ID" in pkg_properties:
			pkg_properties["RIGHT"].append("INTERNET")
			pkg_properties["RIGHT"].append("ACCESS_NETWORK_STATE")
		
		
		debug.print_element("pkg", "absractionFile", "<==", "dynamic file")
		# Create path :
		tools.create_directory_of_file(self.file_final_abstraction)
		# Create file :
		# java ==> done by ewol wrapper ... (and compiled in the normal compilation system ==> must be find in the dependency list of jar ...
		
		tools.create_directory_of_file(target_outpath + "/res/drawable/icon.png");
		if     "ICON" in pkg_properties.keys() \
		   and pkg_properties["ICON"] != "":
			image.resize(pkg_properties["ICON"], target_outpath + "/res/drawable/icon.png", 256, 256)
		else:
			# to be sure that we have all time a resource ...
			tmpFile = open(target_outpath + "/res/drawable/plop.txt", 'w')
			tmpFile.write('plop\n')
			tmpFile.flush()
			tmpFile.close()
		
		if pkg_properties["ANDROID_MANIFEST"]!="":
			debug.print_element("pkg", "AndroidManifest.xml", "<==", pkg_properties["ANDROID_MANIFEST"])
			tools.copy_file(pkg_properties["ANDROID_MANIFEST"], target_outpath + "/AndroidManifest.xml", force=True)
		else:
			debug.error("missing parameter 'ANDROID_MANIFEST' in the properties ... ")
		
		#add properties on wallpaper :
		# myModule.add_pkg("ANDROID_WALLPAPER_PROPERTIES", ["list", key, title, summary, [["key","value display"],["key2","value display 2"]])
		# myModule.add_pkg("ANDROID_WALLPAPER_PROPERTIES", ["list", "testpattern", "Select test pattern", "Choose which test pattern to display", [["key","value display"],["key2","value display 2"]]])
		# myModule.add_pkg("ANDROID_WALLPAPER_PROPERTIES", ["bool", key, title, summary, ["enable string", "disable String"])
		# myModule.add_pkg("ANDROID_WALLPAPER_PROPERTIES", ["bool", "movement", "Motion", "Apply movement to test pattern", ["Moving test pattern", "Still test pattern"]
		#copy needed resources :
		for res_source, res_dest in pkg_properties["ANDROID_RESOURCES"]:
			if res_source == "":
				continue
			tools.copy_file(res_source , target_outpath + "/res/" + res_dest + "/" + os.path.basename(res_source), force=True)
		
		
		# Doc :
		# http://asantoso.wordpress.com/2009/09/15/how-to-build-android-application-package-apk-from-the-command-line-using-the-sdk-tools-continuously-integrated-using-cruisecontrol/
		debug.print_element("pkg", "R.java", "<==", "Resources files")
		tools.create_directory_of_file(target_outpath + "/src/noFile")
		android_tool_path = self.path_sdk + "/build-tools/"
		# find android tool version
		dirnames = tools.get_list_sub_path(android_tool_path)
		if len(dirnames) == 0:
			debug.warning("This does not comport directory: '" + android_tool_path + "'")
			debug.error("An error occured when getting the tools for android")
		elif len(dirnames) > 1:
			dirnames = sorted(dirnames, reverse=True)
			debug.debug("sort tools directory: '" + str(dirnames) + "' ==> select : " + str(dirnames[0]))
		android_tool_path += dirnames[0] + "/"
		
		# this is to create resource file for android ... (we did not use aset in jar with ewol ...
		adModResoucepath = ""
		if "ADMOD_ID" in pkg_properties:
			adModResoucepath = " -S " + self.path_sdk + "/extras/google/google_play_services/libproject/google-play-services_lib/res/ "
		cmdLine = android_tool_path + "aapt p -f " \
		          + "-M " + target_outpath + "/AndroidManifest.xml " \
		          + "-F " + target_outpath + "/resources.res " \
		          + "-I " + self.path_sdk + "/platforms/android-" + str(self.board_id) + "/android.jar "\
		          + "-S " + target_outpath + "/res/ " \
		          + adModResoucepath \
		          + "-J " + target_outpath + "/src/ "
		multiprocess.run_command(cmdLine)
		
		tools.create_directory_of_file(target_outpath + "/build/classes/noFile")
		debug.print_element("pkg", "*.class", "<==", "*.java")
		#generate android java files:
		filesString=""
		
		"""
		old : 
		if "ADMOD_ID" in pkg_properties:
			# TODO : check this I do not think it is really usefull ... ==> write for IDE only ...
			filesString += self.path_sdk + "/extras/google/google_play_services/libproject/google-play-services_lib/src/android/UnusedStub.java "
		if len(pkg_properties["ANDROID_WALLPAPER_PROPERTIES"])!=0:
			filesString += self.path_java_project + pkg_name_application_name + "Settings.java "
		
		adModJarFile = ""
		if "ADMOD_ID" in pkg_properties:
			adModJarFile = ":" + self.path_sdk + "/extras/google/google_play_services/libproject/google-play-services_lib/libs/google-play-services.jar"
		
		cmdLine = "javac " \
		          + "-d " + self.get_staging_path(pkg_name) + "/build/classes " \
		          + "-classpath " + self.path_sdk + "/platforms/android-" + str(self.board_id) + "/android.jar" \
		          + adModJarFile + " " \
		          + filesString \
		          + self.file_final_abstraction + " "  \
		          + self.get_staging_path(pkg_name) + "/src/R.java "
		multiprocess.run_command(cmdLine)
		"""
		debug.verbose("heritage .so=" + str(tools.filter_extention(heritage_list.src['dynamic'], ["so"])))
		debug.verbose("heritage .jar=" + str(tools.filter_extention(heritage_list.src['src'], ["jar"])))
		
		class_extern = ""
		upper_jar = tools.filter_extention(heritage_list.src['src'], ["jar"])
		#debug.warning("ploppppp = " + str(upper_jar))
		for elem in upper_jar:
			if len(class_extern) > 0:
				class_extern += ":"
			class_extern += elem
		# create enpoint element :
		cmdLine = "javac " \
		          + "-d " + target_outpath + "/build/classes " \
		          + "-classpath " + class_extern + " " \
		          + target_outpath + "/src/R.java "
		multiprocess.run_command(cmdLine)
		
		debug.print_element("pkg", ".dex", "<==", "*.class")
		cmdLine = android_tool_path + "dx " \
		          + "--dex --no-strict " \
		          + "--output=" + target_outpath + "/build/" + pkg_name_application_name + ".dex " \
		          + target_outpath + "/build/classes/ "
		
		if "ADMOD_ID" in pkg_properties:
			cmdLine += self.path_sdk + "/extras/google/google_play_services/libproject/google-play-services_lib/libs/google-play-services.jar "
		# add element to dexification:
		for elem in upper_jar:
			# remove android sdk:
			if elem[-len("android.jar"):] != "android.jar":
				cmdLine += elem + " "
		
		multiprocess.run_command(cmdLine)
		
		debug.print_element("pkg", ".apk", "<==", ".dex, assets, .so, res")
		#builderDebug="-agentlib:jdwp=transport=dt_socket,server=y,address=8050,suspend=y "
		builderDebug=""
		# note : set -u not signed application...
		#+ ":" + self.path_sdk + "/extras/google/google_play_services/libproject/google-play-services_lib/libs/google-play-services.jar "
		cmdLine =   "java -Xmx128M " \
		          + " -classpath " + self.path_sdk + "/tools/lib/sdklib.jar " \
		          + builderDebug \
		          + " com.android.sdklib.build.ApkBuilderMain " \
		          + target_outpath + "/build/" + pkg_name_application_name + "-unalligned.apk " \
		          + " -u " \
		          + " -z " + target_outpath + "/resources.res " \
		          + " -f " + target_outpath + "/build/" + pkg_name_application_name + ".dex " \
		          + " -rf " + target_outpath + "/data "
		multiprocess.run_command(cmdLine)
		
		# doc :
		# http://developer.android.com/tools/publishing/app-signing.html
		# Create a key for signing your application:
		# keytool -genkeypair -v -keystore AndroidKey.jks -storepass Pass__AndroidDebugKey -alias alias__AndroidDebugKey -keypass PassKey__AndroidDebugKey -keyalg RSA -validity 36500
		if self.config["mode"] == "debug":
			debug.print_element("pkg", ".apk(signed debug)", "<==", ".apk (not signed)")
			# verbose mode : 
			#debugOption = "-verbose -certs "
			debugOption = ""
			cmdLine = "jarsigner " \
			    + debugOption \
			    + "-keystore " + tools.get_current_path(__file__) + "/AndroidDebugKey.jks " \
			    + " -sigalg SHA1withRSA -digestalg SHA1 " \
			    + " -storepass Pass__AndroidDebugKey " \
			    + " -keypass PassKey__AndroidDebugKey " \
			    + target_outpath + "/build/" + pkg_name_application_name + "-unalligned.apk " \
			    + " alias__AndroidDebugKey"
			multiprocess.run_command(cmdLine)
			tmpFile = open("tmpPass.boo", 'w')
			tmpFile.write("\n")
			tmpFile.flush()
			tmpFile.close()
		else:
			print("On release mode we need the file :  and key an pasword to sign the application ...")
			debug.print_element("pkg", ".apk(signed debug)", "<==", ".apk (not signed)")
			cmdLine = "jarsigner " \
			    + " -keystore " + pkg_properties["ANDROID_SIGN"] + " " \
			    + " -sigalg SHA1withRSA -digestalg SHA1 " \
			    + target_outpath + "/build/" + pkg_name_application_name + "-unalligned.apk " \
			    + " " + pkg_name_application_name
			multiprocess.run_command(cmdLine)
			cmdLine = "jarsigner " \
			    + " -verify -verbose -certs " \
			    + " -sigalg SHA1withRSA -digestalg SHA1 " \
			    + target_outpath + "/build/" + pkg_name_application_name + "-unalligned.apk "
			multiprocess.run_command(cmdLine)
		
		debug.print_element("pkg", ".apk(aligned)", "<==", ".apk (not aligned)")
		tools.remove_file(target_outpath + "/" + pkg_name_application_name + ".apk")
		# verbose mode : -v
		cmdLine = android_tool_path + "zipalign 4 " \
		          + target_outpath + "/build/" + pkg_name_application_name + "-unalligned.apk " \
		          + target_outpath + "/" + pkg_name_application_name + ".apk "
		multiprocess.run_command(cmdLine)
		
		# copy file in the final stage :
		tools.copy_file(target_outpath + "/" + pkg_name_application_name + ".apk",
		                self.get_final_path() + "/" + pkg_name_application_name + ".apk",
		                force=True)
	
	def install_package(self, pkg_name):
		debug.debug("------------------------------------------------------------------------")
		debug.info("Install package '" + pkg_name + "'")
		debug.debug("------------------------------------------------------------------------")
		pkg_name_application_name = pkg_name
		if self.config["mode"] == "debug":
			pkg_name_application_name += "debug"
		cmdLine = self.path_sdk + "/platform-tools/adb install -r " \
		          + self.get_staging_path(pkg_name) + "/" + pkg_name_application_name + ".apk "
		multiprocess.run_command(cmdLine)
	
	def un_install_package(self, pkg_name):
		debug.debug("------------------------------------------------------------------------")
		debug.info("Un-Install package '" + pkg_name + "'")
		debug.debug("------------------------------------------------------------------------")
		pkg_name_application_name = pkg_name
		if self.config["mode"] == "debug":
			pkg_name_application_name += "debug"
		cmdLine = self.path_sdk + "/platform-tools/adb uninstall " + pkg_name_application_name
		Rmultiprocess.run_command(cmdLine)
	
	def show_log(self, pkg_name):
		debug.debug("------------------------------------------------------------------------")
		debug.info("logcat of android board")
		debug.debug("------------------------------------------------------------------------")
		debug.info("cmd: " + self.path_sdk + "/platform-tools/adb shell logcat ")
		cmdLine = self.path_sdk + "/platform-tools/adb shell logcat "
		multiprocess.run_command_no_lock_out(cmdLine)





#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

from realog import debug
from lutin import system
from lutin import tools
from lutin import env
import os

class System(system.System):
	def __init__(self, target):
		system.System.__init__(self)
		# create some HELP:
		self.set_help("PULSE : The Linux PulseAudio\n Can be install with the package:\n    - libpulse-dev")
		# check if the library exist:
		if     not os.path.isfile("/usr/include/pulse/pulseaudio.h"):
			# we did not find the library reqiested (just return) (automaticly set at false)
			return;
		dst_data = tools.file_read_data("/usr/include/pulse/version.h")
		lines = dst_data.split("\n")
		patern = "#define pa_get_headers_version() (\""      # " #corect edn error parsing
		version = None
		for line in lines:
			if line[:len(patern)] == patern:
				#Find the version line
				offset = len(patern)
				version = ""
				while     offset < len(line) \
				      and line[offset] != '.':
					version += line[offset]
					offset += 1
				offset += 1
				version2 = ""
				while     offset < len(line) \
				      and line[offset] != '.':
					version2 += line[offset]
					offset += 1
				debug.verbose("detect version '" + version + "'")
				break;
		if version == None:
			debug.warning("Can not det version of Pulseaudio ... ==> remove it")
			return
		self.set_version([int(version),int(version2)])
		self.set_valid(True)
		self.add_depend([
		    'c'
		    ])
		if env.get_isolate_system() == False:
			self.add_flag("link-lib", [
			    "pulse-simple",
			    "pulse"
			    ])
		else:
			# todo : create a searcher of the presence of the library:
			"""
			self.add_flag("link-lib", [
			    "-l/lib/pulseaudio/libpulsecommon-" + version + ".0.so"
			    ])
			"""
			self.add_flag("link-lib", [
			    "pulsecommon-" + version + ".0",
			    "pulse-mainloop-glib",
			    "pulse-simple",
			    "pulse"
			    ])
			self.add_flag("link", "-L/usr/lib/pulseaudio")
			self.add_flag("link", "-Wl,-R/usr/lib/pulseaudio")
			self.add_header_file([
			    "/usr/include/pulse/*",
			    ],
			    destination_path="pulse",
			    recursive=True)





#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

from realog import debug
from lutin import target
from lutin import tools
from lutin import host
from lutin import multiprocess
from lutin import depend
from lutin import env
import os
import stat
import shutil

class Target(target.Target):
	def __init__(self, config, sub_name=[]):
		#processor type selection (auto/arm/ppc/x86)
		if config["arch"] == "auto":
			config["arch"] = "x86"
		#bus size selection (auto/32/64)
		if config["bus-size"] == "auto":
			config["bus-size"] = str(host.BUS_SIZE)
		#config["bus-size"] = str(32)
		if config["compilator"] != "clang":
			debug.warning("compilator is not clang ==> force it...")
			config["compilator"] = "clang"
		# http://biolpc22.york.ac.uk/pub/linux-mac-cross/
		# http://devs.openttd.org/~truebrain/compile-farm/apple-darwin9.txt
		target.Target.__init__(self, ["MacOs"] + sub_name, config, "")
		
		#self.path_bin="MacOS"
		#self.path_lib="lib"
		#self.path_data="Resources"
		#self.path_doc="doc"
		
		self.suffix_lib_static='.a'
		self.suffix_lib_dynamic='.dylib'
		#self.suffix_binary=''
		#self.suffix_package=''
		
		self.pkg_path_data = "Resources"
		self.pkg_path_bin = "MacOS"
		self.pkg_path_lib = "lib"
		self.pkg_path_license = "license"
		if env.get_isolate_system() == True:
			self.sysroot = "--sysroot=/aDirectoryThatDoesNotExist/"
		
		self.add_flag("link-lib", [
		    "dl"
		    ])
		self.add_flag("link", [
		    "-rdynamic"
		    ])
		
	"""
	def get_staging_path(self, binary_name):
		return tools.get_run_path() + self.path_out + self.path_staging + "/" + binary_name + ".app/Contents/"
	
	def get_staging_path_data(self, binary_name):
		return self.get_staging_path(binary_name) + self.path_data + "/"
	"""
	
	def make_package_binary(self, pkg_name, pkg_properties, base_pkg_path, heritage_list, static):
		debug.debug("------------------------------------------------------------------------")
		debug.info("Generate package '" + pkg_name + "' v" + tools.version_to_string(pkg_properties["VERSION"]))
		debug.debug("------------------------------------------------------------------------")
		#output path
		target_outpath = os.path.join(self.get_staging_path(pkg_name), pkg_name + ".app/Contents")
		tools.create_directory_of_file(target_outpath)
		
		## Create share datas:
		ret_share = self.make_package_binary_data(target_outpath, pkg_name, base_pkg_path, heritage_list, static)
		
		## copy binary files:
		ret_bin = self.make_package_binary_bin(target_outpath, pkg_name, base_pkg_path, heritage_list, static)
		
		## Create libraries:
		ret_lib = self.make_package_binary_lib(target_outpath, pkg_name, base_pkg_path, heritage_list, static)
		
		## Create generic files:
		ret_file = self.make_package_generic_files(target_outpath, pkg_properties, pkg_name, base_pkg_path, heritage_list, static)
		
		## end of the package generation
		build_package_path_done = os.path.join(self.get_build_path(pkg_name), "generatePackageDone.txt")
		#Check date between the current file "list of action to generate package and the end of package generation
		need_generate_package = depend.need_re_package(build_package_path_done, [__file__], True)
		
		## create the package:
		if    ret_share \
		   or ret_bin \
		   or ret_lib \
		   or ret_file \
		   or need_generate_package:
			
			## Create icon (no convertion ==> TODO: must test if png is now supported):
			if     "ICON" in pkg_properties.keys() \
			   and pkg_properties["ICON"] != "":
				tools.copy_file(pkg_properties["ICON"], os.path.join(target_outpath, "icon.icns"), force=True)
			
			## Create info.plist file:
			# http://www.sandroid.org/imcross/#Deployment
			data_file  = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
			data_file += "<!DOCTYPE plist PUBLIC \"-//Apple Computer//DTD PLIST 1.0//EN\" \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">\n"
			data_file += "<plist version=\"1.0\">\n"
			data_file += "    <dict>\n"
			data_file += "        <key>CFBundleExecutableFile</key>\n"
			data_file += "        <etk/String.hpp>"+pkg_name+"</string>\n"
			data_file += "        <key>CFBundleName</key>\n"
			data_file += "        <etk/String.hpp>"+pkg_name+"</string>\n"
			data_file += "        <key>CFBundleIdentifier</key>\n"
			data_file += "        <etk/String.hpp>" + pkg_properties["COMPAGNY_TYPE"] + "." + pkg_properties["COMPAGNY_NAME2"] + "." + pkg_name + "</string>\n"
			data_file += "        <key>CFBundleSignature</key>\n"
			data_file += "        <etk/String.hpp>????</string>\n"
			data_file += "        <key>CFBundleIconFile</key>\n"
			data_file += "        <etk/String.hpp>icon.icns</string>\n"
			data_file += "    </dict>\n"
			data_file += "</plist>\n"
			data_file += "\n\n"
			tools.file_write_data(os.path.join(target_outpath, "Info.plist"),
			                      data_file,
			                      only_if_new=True)
			
			## Create PkgInfo file:
			tools.file_write_data(os.path.join(target_outpath, "PkgInfo"),
			                      "APPL????",
			                      only_if_new=True)
			
			## Create a simple interface to localy install the aplication for the shell (a shell command line interface):
			data_file  = "#!/bin/bash\n"
			data_file += "# Simply open the real application in the correct way (a link does not work ...)\n"
			data_file += "/Applications/" + pkg_name + ".app/Contents/MacOS/" + pkg_name + " $*\n"
			tools.file_write_data(os.path.join(target_outpath, "shell", pkg_name),
			                      data_file,
			                      only_if_new=True)
			
			## Create the disk image of the application:
			debug.info("Generate disk image for '" + pkg_name + "'")
			output_file_name = os.path.join(self.get_final_path(), pkg_name + ".dmg")
			cmd = "hdiutil create -volname "
			cmd += pkg_name + " -srcpath "
			cmd += os.path.join(tools.get_run_path(), self.path_out,  self.path_staging, pkg_name + ".app")
			cmd += " -ov -format UDZO "
			cmd += output_file_name
			tools.create_directory_of_file(output_file_name)
			multiprocess.run_command_direct(cmd)
			debug.info("disk image: " + output_file_name)
			
			## user information:
			#debug.info("You can have an shell interface by executing : ")
			#debug.info("    sudo cp " + shell_file_name + " /usr/local/bin")
			tools.file_write_data(build_package_path_done, "done...")
	
	def install_package(self, pkg_name):
		debug.debug("------------------------------------------------------------------------")
		debug.info("Install package '" + pkg_name + "'")
		debug.debug("------------------------------------------------------------------------")
		debug.info("copy " + os.path.join(self.get_staging_path(pkg_name), pkg_name + ".app") + " in /Applications/")
		if os.path.exists("/Applications/" + pkg_name + ".app") == True:
			shutil.rmtree("/Applications/" + pkg_name + ".app")
		# copy the application in the basic application path : /Applications/xxx.app
		shutil.copytree(os.path.join(self.get_staging_path(pkg_name), pkg_name + ".app"), os.path.join("/Applications", pkg_name + ".app"))
	
	def un_install_package(self, pkg_name):
		debug.debug("------------------------------------------------------------------------")
		debug.info("Un-Install package '" + pkg_name + "'")
		debug.debug("------------------------------------------------------------------------")
		debug.info("remove OLD application /Applications/" + pkg_name + ".app")
		# Remove the application in the basic application path : /Applications/xxx.app
		if os.path.exists("/Applications/" + pkg_name + ".app") == True:
			shutil.rmtree("/Applications/" + pkg_name + ".app")
	
	def run(self, pkg_name, option_list, binary_name = None):
		debug.debug("------------------------------------------------------------------------")
		debug.info("-- Run package '" + pkg_name + "'")
		debug.debug("------------------------------------------------------------------------")
		appl_path = os.path.join(self.get_staging_path(pkg_name), pkg_name + ".app" , "Contents", "MacOS", pkg_name)
		appl_path = os.path.join(self.get_staging_path(pkg_name), pkg_name + ".app", "Contents", "MacOS", pkg_name)
		cmd = appl_path + " "
		for elem in option_list:
			cmd += elem + " "
		multiprocess.run_command_no_lock_out(cmd)
		debug.debug("------------------------------------------------------------------------")
		debug.info("-- Run package '" + pkg_name + "' Finished")
		debug.debug("------------------------------------------------------------------------")









#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

from realog import debug
from lutin import system
from lutin import tools
from lutin import env
import os

class System(system.System):
	def __init__(self, target):
		system.System.__init__(self)
		# create some HELP:
		self.set_help("C: Generic C library")
		self.add_flag("c-remove","-nodefaultlibs")
		self.set_valid(True)
		if env.get_isolate_system() == False:
			# We must have it ... all time
			pass
		else:
			# grep "This file is part of the GNU C Library" /usr/include/*
			self.add_header_file([
			    '/usr/include/aio.h*',
			    '/usr/include/aliases.h*',
			    '/usr/include/alloca.h*',
			    '/usr/include/ansidecl.h*',
			    '/usr/include/argp.h*',
			    '/usr/include/argz.h*',
			    '/usr/include/ar.h*',
			    '/usr/include/assert.h*',
			    '/usr/include/byteswap.h*',
			    '/usr/include/complex.h*',
			    '/usr/include/cpio.h*',
			    '/usr/include/ctype.h*',
			    '/usr/include/dirent.h*',
			    '/usr/include/dlfcn.h*',
			    '/usr/include/elf.h*',
			    '/usr/include/endian.h*',
			    '/usr/include/envz.h*',
			    '/usr/include/err.h*',
			    '/usr/include/errno.h*',
			    '/usr/include/error.h*',
			    '/usr/include/execinfo.h*',
			    '/usr/include/fcntl.h*',
			    '/usr/include/features.h*',
			    '/usr/include/fenv.h*',
			    '/usr/include/fmtmsg.h*',
			    '/usr/include/fnmatch.h*',
			    '/usr/include/fpu_control.h*',
			    '/usr/include/fts.h*',
			    '/usr/include/ftw.h*',
			    '/usr/include/gconv.h*',
			    '/usr/include/getopt.h*',
			    '/usr/include/glob.h*',
			    '/usr/include/gnu-versions.h*',
			    '/usr/include/grp.h*',
			    '/usr/include/gshadow.h*',
			    '/usr/include/iconv.h*',
			    '/usr/include/ieee754.h*',
			    '/usr/include/ifaddrs.h*',
			    '/usr/include/inttypes.h*',
			    '/usr/include/langinfo.h*',
			    '/usr/include/libgen.h*',
			    '/usr/include/libintl.h*',
			    '/usr/include/libio.h*',
			    '/usr/include/limits.h*',
			    '/usr/include/link.h*',
			    '/usr/include/locale.h*',
			    '/usr/include/malloc.h*',
			    '/usr/include/mcheck.h*',
			    '/usr/include/memory.h*',
			    '/usr/include/mntent.h*',
			    '/usr/include/monetary.h*',
			    '/usr/include/mqueue.h*',
			    '/usr/include/netdb.h*',
			    '/usr/include/nl_types.h*',
			    '/usr/include/nss.h*',
			    '/usr/include/obstack.h*',
			    '/usr/include/printf.h*',
			    '/usr/include/pthread.h*',
			    '/usr/include/pty.h*',
			    '/usr/include/pwd.h*',
			    '/usr/include/re_comp.h*',
			    '/usr/include/regex.h*',
			    '/usr/include/regexp.h*',
			    '/usr/include/sched.h*',
			    '/usr/include/search.h*',
			    '/usr/include/semaphore.h*',
			    '/usr/include/setjmp.h*',
			    '/usr/include/sgtty.h*',
			    '/usr/include/shadow.h*',
			    '/usr/include/signal.h*',
			    '/usr/include/spawn.h*',
			    '/usr/include/stdc-predef.h*',
			    '/usr/include/stdint.h*',
			    '/usr/include/stdio_ext.h*',
			    '/usr/include/stdio.h*',
			    '/usr/include/stdlib.h*',
			    '/usr/include/string.h*',
			    '/usr/include/strings.h*',
			    '/usr/include/stropts.h*',
			    '/usr/include/tar.h*',
			    '/usr/include/termios.h*',
			    '/usr/include/tgmath.h*',
			    '/usr/include/thread_db.h*',
			    '/usr/include/time.h*',
			    '/usr/include/uchar.h*',
			    '/usr/include/ucontext.h*',
			    '/usr/include/ulimit.h*',
			    '/usr/include/unistd.h*',
			    '/usr/include/utime.h*',
			    '/usr/include/utmp.h*',
			    '/usr/include/utmpx.h*',
			    '/usr/include/values.h*',
			    '/usr/include/wchar.h*',
			    '/usr/include/wctype.h*',
			    '/usr/include/wordexp.h*',
			    '/usr/include/xlocale.h*',
			    '/usr/include/Availability*',
			    '/usr/include/_types.h*',
			    '/usr/include/_wctype.h*',
			    '/usr/include/runetype.h*',
			    '/usr/include/_locale.h*',
			    '/usr/include/_xlocale.h*',
			    '/usr/include/gethostuuid.h*',
			    ],
			    destination_path="")
			self.add_header_file([
			    '/usr/include/poll.h*',
			    '/usr/include/unistdio.h*',
			    '/usr/include/syslog.h*',
			    '/usr/include/_G_config.h*',
			    ],
			    destination_path="")
			self.add_header_file([
			    "/usr/include/machine/*",
			    ],
			    destination_path="machine",
			    recursive=True)
			self.add_header_file([
			    "/usr/include/xlocale/*",
			    ],
			    destination_path="xlocale",
			    recursive=True)
			self.add_header_file([
			    "/usr/include/i386/*",
			    ],
			    destination_path="i386",
			    recursive=True)
			self.add_header_file([
			    "/usr/include/_types/*",
			    ],
			    destination_path="_types",
			    recursive=True)
			self.add_header_file([
			    "/usr/include/libkern/*",
			    ],
			    destination_path="libkern",
			    recursive=True)
			self.add_header_file([
			    "/usr/include/mach/*",
			    ],
			    destination_path="mach",
			    recursive=True)
			self.add_header_file([
			    "/usr/include/sys/*",
			    ],
			    destination_path="sys",
			    recursive=True)
			self.add_header_file([
			    "/usr/include/bits/*",
			    ],
			    destination_path="bits",
			    recursive=True)
			self.add_header_file([
			    "/usr/include/gnu/*",
			    ],
			    destination_path="gnu",
			    recursive=True)
			self.add_header_file([
			    "/usr/include/linux/*",
			    ],
			    destination_path="linux",
			    recursive=True)
			self.add_header_file([
			    "/usr/include/asm/*",
			    ],
			    destination_path="asm",
			    recursive=True)
			self.add_header_file([
			    "/usr/include/asm-generic/*",
			    ],
			    destination_path="asm-generic",
			    recursive=True)
			self.add_header_file([
			    "/usr/include/netinet/*",
			    ],
			    destination_path="netinet",
			    recursive=True)
			self.add_header_file([
			    "/usr/include/net/*",
			    ],
			    destination_path="net",
			    recursive=True)
			self.add_flag("link", "-B/usr/lib")




#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

##
## Executable/binary builder
##
from lutin import multiprocess
from lutin import tools
from realog import debug
from lutin import depend
from lutin import env
import os

##
## Initialize the builder, if needed ... to get dependency between builder (for example)
##
def init():
	pass

##
## Get the current builder type.
## Return the type of builder
##
def get_type():
	return "linker"


##
## @brief Get builder input file type
## @return List of extention supported
##
def get_input_type():
	return ["o"]

##
## @brief Get builder output file type
## @return List of extention supported
##
def get_output_type():
	return ["", "exe", "bin"]

##
## @brief Get builder support multi-threading or not
## @return True Multithreading supported
## @return False Multithreading NOT supported
##
def get_support_multithreading():
	return False

##
## @brief Commands for running gcc to link an executable.
##
def link(file, binary, target, depancy, flags, name, basic_path, static = False):
	file_src = file
	file_dst = target.get_build_file_bin(name, static)
	file_depend = file_dst + target.suffix_dependence
	file_cmd = file_dst + target.suffix_cmd_line
	file_warning = file_dst + target.suffix_warning
	
	debug.extreme_verbose("list files = " + str(depancy.src))
	list_static = []
	list_dynamic = []
	if static == True:
		#get all parent static libs
		list_static = depancy.src['static']
		# get only parent shared that is not static
		for elem in depancy.src['dynamic']:
			lib_name = elem[:-len(target.suffix_lib_dynamic)] + target.suffix_lib_static
			if lib_name not in depancy.src['static']:
				list_dynamic.append(elem)
	else:
		#get all parent dynamic libs
		list_dynamic = depancy.src['dynamic']
		# get only parent shared that is not static
		for elem in depancy.src['static']:
			lib_name = elem[:-len(target.suffix_lib_static)] + target.suffix_lib_dynamic
			if lib_name not in depancy.src['dynamic']:
				list_static.append(elem)
	# set ccache interface:
	compilator_ccache = ""
	if env.get_ccache() == True:
		compilator_ccache = "ccache"
	#create comand line:
	cmd = [compilator_ccache]
	# a specific case to not depend on the libstdc++ automaticly added by the G++ or clang++ compilator ==> then need to compile with GCC or CLANG if use libcxx from llvm or other ...
	if     "need-libstdc++" in depancy.flags \
	   and depancy.flags["need-libstdc++"] == True:
		cmd.append(target.xx)
	else:
		cmd.append(target.cc)
	
	try:
		cmd.append(target.arch)
	except:
		pass
	try:
		cmd.append(target.sysroot)
	except:
		pass
	try:
		cmd.append(["-o", file_dst])
	except:
		pass
	try:
		cmd.append(file_src)
	except:
		pass
	try:
		cmd.append(depancy.src['src'])
	except:
		pass
	try:
		cmd.append(list_static)
	except:
		pass
	try:
		for elem in list_dynamic:
			lib_path = os.path.dirname(elem)
			lib_name = elem[len(lib_path)+len(target.prefix_lib)+1:-len(target.suffix_lib_dynamic)]
			cmd.append("-L" + lib_path)
			cmd.append("-l" + lib_name)
		if     target.name != "MacOs" \
		   and target.name != "Android":
			if len(list_dynamic) > 0:
				cmd.append("-Wl,-R$ORIGIN/../lib/")
	except:
		pass
	cmd.append("-Wl,-rpath,\"$ORIGIN/../lib\"")
	try:
		cmd.append(flags["local"]["link"])
	except:
		pass
	try:
		cmd.append(depancy.flags["link"])
	except:
		pass
	try:
		cmd.append(target.global_flags["link"])
	except:
		pass
	for view in ["local", "export"]:
		if view not in flags:
			continue
		for type in ["link-lib"]:
			if type in flags[view]:
				cmd.append([("-l" + sss).replace("-l/", "/") for sss in flags[view][type] ])
		for type in ["link-bin"]:
			if type in flags[view]:
				cmd.append(flags[view][type])
	for type in ["link-lib"]:
		if type in depancy.flags:
			cmd.append([("-l" + sss).replace("-l/", "/") for sss in depancy.flags[type] ])
	for type in ["link-lib"]:
		if type in target.global_flags:
			cmd.append([("-l" + sss).replace("-l/", "/") for sss in target.global_flags[type] ])
	for type in ["link-bin"]:
		if type in target.global_flags:
			cmd.append(target.global_flags[type])
		if type in depancy.flags:
			cmd.append(depancy.flags[type])
	cmd_line = tools.list_to_str(cmd)
	# check the dependency for this file :
	if     depend.need_re_package(file_dst, file_src, True, file_cmd=file_cmd, cmd_line=cmd_line) == False \
	   and depend.need_re_package(file_dst, depancy.src, False, file_cmd=file_cmd, cmd_line=cmd_line) == False:
		return file_dst
	tools.create_directory_of_file(file_dst)
	debug.print_element("Executable", name, "==>", os.path.relpath(file_dst))
	
	multiprocess.run_command(cmd_line, store_output_file=file_warning)
	if    target.config["mode"] == "release"\
	   or env.get_force_strip_mode() == True:
		# get the file size of the non strip file
		originSize = tools.file_size(file_dst);
		debug.print_element("Executable(strip)", name, "", "")
		cmd_lineStrip=tools.list_to_str([
			target.strip,
			file_dst])
		multiprocess.run_command(cmd_lineStrip, store_output_file=file_warning)
		# get the stip size of the binary
		strip_size = tools.file_size(file_dst)
		debug.debug("file reduce size : " + str(originSize/1024) + "ko ==> " + str(strip_size/1024) + "ko")
	# write cmd line only after to prevent errors ...
	tools.store_command(cmd_line, file_cmd)
	



#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

from realog import debug
from lutin import system
from lutin import tools
from lutin import env
import os

class System(system.System):
	def __init__(self, target):
		system.System.__init__(self)
		# create some HELP:
		self.set_help("JACK : Jack Low-Latency Audio Server\n Can be install with the package:\n    - libjack-jackd2-dev (new)\n    - libjack-dev (old)")
		# check if the library exist:
		if     not os.path.isfile("/usr/include/jack/jack.h"):
			# we did not find the library reqiested (just return) (automaticly set at false)
			return;
		self.set_valid(True)
		self.add_depend([
		    'uuid',
		    'c'
		    ])
		self.add_flag("link-lib", "jack")
		if env.get_isolate_system() == True:
			self.add_header_file([
			    "/usr/include/jack/*",
			    ],
			    destination_path="jack",
			    recursive=True)





#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

import sys
import os
import inspect
import fnmatch
import datetime
# Local import
from realog import debug
from . import heritage
from . import tools
from . import module
from . import system
from . import multiprocess
from . import env
##
## @brief Target class represent the buyild environement for a specific platform like Linux, or Android ....
##
class Target:
	##
	## @brief contructor
	## @param[in] name ([string,...]) Name of the target
	## @param[in] config (dict) User configuration
	## @param[in] arch (string) specific parameter for gcc -arch element
	##
	def __init__(self, name, config, arch):
		if tools.get_type_string(name) != "list":
			debug.error("You must define a name in a list ...")
		if len(name) < 1:
			debug.error("You must define a name for your target ...")
		## configuration of the build
		self.config = config
		
		if self.config["bus-size"] == "auto":
			debug.error("system error ==> must generate the default 'bus-size' config")
		if self.config["arch"] == "auto":
			debug.error("system error ==> must generate the default 'bus-size' config")
		
		debug.debug("config=" + str(config))
		if arch != "":
			self.arch = "-arch " + arch
		else:
			self.arch = ""
		
		self.end_generate_package = config["generate-package"]
		# todo : remove this :
		self._name = name[-1]
		self._config_based_on = name
		debug.info("=================================");
		debug.info("== Target='" + self._name + "' " + self.config["bus-size"] + " bits for arch '" + self.config["arch"] + "'");
		debug.info("== Target list=" + str(self._config_based_on))
		debug.info("=================================");
		
		self.set_cross_base()
		
		###############################################################################
		# Target global variables.
		###############################################################################
		self.global_include_cc=[]
		self.global_flags={}
		
		self.global_libs_ld=[]
		self.global_libs_ld_shared=[]
		
		self.suffix_cmd_line='.cmd'
		self.suffix_warning='.warning'
		self.suffix_dependence='.d'
		self.suffix_obj='.o'
		self.prefix_lib='lib'
		self.suffix_lib_static='.a'
		self.suffix_lib_dynamic='.so'
		self.suffix_binary=''
		self.suffix_package='.deb'
		
		self.path_generate_code="/generate_header"
		self.path_arch = "/" + self._name
		
		for elem in self._config_based_on:
			self.add_flag("c", '-D__TARGET_OS__' + elem)
		self.add_flag("c", [
		    '-D__TARGET_ARCH__' + self.config["arch"],
		    '-D__TARGET_ADDR__' + self.config["bus-size"] + 'BITS',
		    '-D_REENTRANT'
		    ])
		if     self.config["compilator"] == "clang" \
		   and self.xx_version >= 4002001: # >= 4.2.1
			self.add_flag("c++", "-Wno-undefined-var-template")
		self.add_flag("c", "-nodefaultlibs")
		self.add_flag("c++", "-nostdlib")
		# this disable the need to have the __cxa_guard_release
		self.add_flag("c++", "-fno-threadsafe-statics")
		#self.add_flag("c", "-nostdinc") #ignores standard C include directories
		#self.add_flag("c++", "-nostdinc++") #ignores standard C++ include directories
		self.add_flag("ar", 'rcs')
		
		if self._name == "Windows":
			self.add_flag("c++", [
			    '-static-libgcc',
			    '-static-libstdc++'
			    ])
		if "debug" == self.config["mode"]:
			self.add_flag("c", [
			    "-g",
			    "-DDEBUG"
			    ])
			if env.get_force_optimisation() == False:
				self.add_flag("c", "-O0")
			else:
				self.add_flag("c", "-O3")
		else:
			self.add_flag("c", [
			    "-DNDEBUG",
			    "-O3"
			    ])
		
		## To add code coverate on build result system
		if self.config["gcov"] == True:
			if self.config["compilator"] == "clang":
				self.add_flag("c", [
				    "--coverage"
				    ])
				self.add_flag("link", [
				    "--coverage"
				    ])
			else:
				self.add_flag("c", [
				    "-fprofile-arcs",
				    "-ftest-coverage"
				    ])
				self.add_flag("link", [
				    "-lgcov",
				    "--coverage"
				    ])
		
		self._update_path_tree()
		self.path_bin="bin"
		self.path_lib="lib"
		self.path_data="share"
		self.path_doc="doc"
		self.path_include="include"
		self.path_temporary_generate="generate"
		self.path_object="obj"
		
		
		self.build_done=[]
		self.build_tree_done=[]
		self.module_list=[]
		# output staging files list :
		self.list_final_file=[]
		
		self.sysroot=""
		
		self.action_on_state={}
		
		# set some default package path
		self.pkg_path_version_file = "version.txt"
		self.pkg_path_maintainer_file = "maintainer.txt"
		self.pkg_path_application_name_file = "appl_name.txt"
		self.pkg_path_application_description_file = "appl_description.txt"
		self.pkg_path_readme_file = "readme.txt"
		self.pkg_path_change_log_file = "changelog.txt"
		
		# special case for IOS (example) no build dynamicly ...
		self.support_dynamic_link = True
	
	##
	## @brief Generate a string representing the class (for str(xxx))
	## @param[in] self (handle) Class handle
	## @return (string) string of str() convertion
	##
	def __repr__(self):
		return "{lutin.Target}"
	
	##
	## @brief Get the type of the target: ["Linux, ...]
	## @param[in] self (handle) Class handle
	## @return ([string,...]) The current target name and other sub name type (ubuntu ...)
	##
	def get_type(self):
		return self._config_based_on
	
	##
	## @brief Add a type that the model is based on
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of that the element is based on ...
	##
	def add_type(self, name):
		self._config_based_on.append(name)
	
	##
	## @brief Get the name of the target: Linux, Windows, ...
	## @param[in] self (handle) Class handle
	## @return (string) Name of the target
	##
	def get_name(self):
		return self._name
	
	##
	## @brief Get build mode of the target: ["debug", "release"]
	## @param[in] self (handle) Class handle
	## @return (string) The current target build mode.
	##
	def get_mode(self):
		return self.config["mode"]
	
	##
	## @brief Get build for a simulator (Ios and Android for example)
	## @param[in] self (handle) Class handle
	## @return (bool) sumulation requested
	##
	def get_simulation(self):
		return self.config["simulation"]
	
	##
	## @brief Get compilator name (clang / gcc)
	## @param[in] self (handle) Class handle
	## @return (bool) name of the compilator requested
	##
	def get_compilator(self):
		return self.config["compilator"]
	
	##
	## @brief Get architecture name (x86 / arm / ...)
	## @param[in] self (handle) Class handle
	## @return (bool) name of the arch requested
	##
	def get_arch(self):
		return self.config["arch"]
	##
	## @brief Get architecture name (x86 / arm / ...)
	## @param[in] self (handle) Class handle
	## @return (bool) name of the arch requested
	##
	def get_bus_size(self):
		return self.config["bus-size"]
	
	##
	## @brief Add global target flags
	## @param[in] self (handle) Class handle
	## @param[in] type (string) inclusion group name 'c', 'c++', 'java' ...
	## @param[in] in_list ([string,...] or string) List of path to include
	## @return None
	##
	def add_flag(self, in_type, in_list):
		tools.list_append_to_2(self.global_flags, in_type, in_list)
	
	##
	## @brief Update basic tree path positions on the build tree
	## @param[in] self (handle) Class handle
	## @return None
	##
	def _update_path_tree(self):
		self.path_out = os.path.join("out", self._name + "_" + self.config["arch"] + "_" + self.config["bus-size"], self.config["mode"])
		self.path_final = os.path.join("final", self.config["compilator"])
		self.path_staging = os.path.join("staging", self.config["compilator"])
		self.path_staging_tmp = os.path.join("staging_tmp", self.config["compilator"])
		self.path_build = os.path.join("build", self.config["compilator"])
	
	# TODO: Remove this from here ==> this is a tools
	##
	## @brief create a string version number with the associated list values
	## @param[in] self (handle) Class handle
	## @param[in] data ([int|string,...]) version basic number
	## @return (string) version number
	##
	def create_number_from_version_string(self, data):
		tmp_data = data.split("-")
		if len(tmp_data) > 1:
			data = tmp_data[0]
		list = data.split(".")
		if len(list) == 1:
			list.append("0")
		if len(list) == 2:
			list.append("0")
		if len(list) > 3:
			list = list[:3]
		out = 0;
		offset = 1000**(len(list)-1)
		for elem in list:
			out += offset*int(elem)
			debug.verbose("get : " + str(int(elem)) + " tmp" + str(out))
			offset /= 1000
		return out
	
	##
	## @brief Configure the cross toolchain
	## @param[in] self (handle) Class handle
	## @param[in] cross (string) Path of the cross toolchain
	## @return None
	##
	def set_cross_base(self, cross=""):
		self.cross = cross
		debug.debug("== Target='" + self.cross + "'");
		self.java = "javac"
		self.javah = "javah"
		self.jar = "jar"
		self.ar = self.cross + "ar"
		self.ranlib = self.cross + "ranlib"
		if self.config["compilator"] == "clang":
			self.cc = self.cross + "clang"
			self.xx = self.cross + "clang++"
			#self.ar=self.cross + "llvm-ar"
			self.ranlib=""
		else:
			self.cc = self.cross + "gcc"
			self.xx = self.cross + "g++"
			#self.ar=self.cross + "ar"
			#self.ranlib=self.cross + "ranlib"
		if self.config["compilator-version"] != "":
			self.cc = self.cc + "-" + self.config["compilator-version"]
			self.xx = self.xx + "-" + self.config["compilator-version"]
		
		#get g++ compilation version :
		ret = multiprocess.run_command_direct(self.xx + " -dumpversion");
		if ret == False:
			debug.error("Can not get the g++/clang++ version ...")
		self.xx_version = self.create_number_from_version_string(ret)
		debug.debug(self.config["compilator"] + "++ version=" + str(ret) + " number=" + str(self.xx_version))
		
		self.ld = self.cross + "ld"
		self.nm = self.cross + "nm"
		self.strip = self.cross + "strip"
		self.dlltool = self.cross + "dlltool"
		self._update_path_tree()
		
		#some static libraries that is sometime needed when not use stdlib ...
		ret = multiprocess.run_command_direct(self.xx + " -print-file-name=libgcc.a");
		if ret == False:
			debug.error("Can not get the g++/clang++ libgcc.a ...")
		self.stdlib_name_libgcc = ret;
		ret = multiprocess.run_command_direct(self.xx + " -print-file-name=libsupc++.a");
		if ret == False:
			debug.error("Can not get the g++/clang++ libsupc++.a ...")
		self.stdlib_name_libsupc = ret;
	
	##
	## @brief Get the current build mode
	## @param[in] self (handle) Class handle
	## @return Build mode value [debug,release]
	##
	def get_build_mode(self):
		return self.config["mode"]
	
	def get_full_name_source(self, basePath, file):
		if file[0] == '/':
			if tools.os.path.isfile(file):
				return file
		return basePath + "/" + file
	
	def get_full_name_cmd(self, module_name, basePath, file):
		if file[0] == '/':
			if tools.os.path.isfile(file):
				return file + self.suffix_cmd_line
		return self.get_build_path_object(module_name) + "/" + file + self.suffix_cmd_line
	
	def get_full_name_warning(self, module_name, basePath, file):
		return self.get_build_path_object(module_name) + "/" + file + self.suffix_warning;
	
	def get_full_name_destination(self, module_name, basePath, file, suffix, remove_suffix=False):
		# special patch for java file:
		if file[-4:] == "java":
			for elem in ["org/", "com/"]:
				pos = file.find(elem);
				if pos > 0:
					file = file[pos:]
		if remove_suffix == True:
			file = file[:file.rfind(".")] + '.'
		else:
			file += "."
		if len(suffix) >= 1:
			suffix = suffix[0]
		else:
			suffix = ""
		return self.get_build_path_object(module_name) + "/" + file + suffix
	
	def get_full_dependency(self, module_name, basePath, file):
		return self.get_build_path_object(module_name) + "/" + file + self.suffix_dependence
	
	##
	## @brief Get the final path ==> contain all the generated packages
	## @param[in] self (handle) Class handle
	## @return (string) The path
	##
	def get_final_path(self):
		return os.path.join(tools.get_run_path(), self.path_out, self.path_final)
	
	##
	## @brief Get the staging path ==> all install stangalone package (no dependency file, no object files, no cmdlines files
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @return (string) The path
	##
	def get_staging_path(self, name, tmp=False):
		if tmp == False:
			return os.path.join(tools.get_run_path(), self.path_out, self.path_staging, name)
		else:
			return os.path.join(tools.get_run_path(), self.path_out, self.path_staging_tmp, name)
	
	##
	## @brief Get the build path ==> dependency file, object files, cmdlines files, generate files, local install headers ...
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @return (string) The path
	##
	def get_build_path(self, name):
		return os.path.join(tools.get_run_path(), self.path_out, self.path_build, name)
	
	##
	## @brief Get the build object path where write .o files
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @return (string) The path
	##
	def get_build_path_object(self, name):
		return os.path.join(self.get_build_path(name), self.path_object)
	
	##
	## @brief Get the build binary path where write .bin, .exe ... files
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @return (string) The path
	##
	def get_build_path_bin(self, name):
		return os.path.join(self.get_build_path(name), self.path_bin)
	
	##
	## @brief Get the shared/static library object path where write .a / .so files
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @return (string) The path
	##
	def get_build_path_lib(self, name):
		return os.path.join(self.get_build_path(name), self.path_lib)
	
	##
	## @brief Get the data path where pre-write the install "data" files
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @return (string) The path
	##
	def get_build_path_data(self, name):
		return os.path.join(self.get_build_path(name), self.path_data, name)
	
	##
	## @brief Get the include path where pre-install "include" headers files
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @return (string) The path
	##
	def get_build_path_include(self, name):
		return os.path.join(self.get_build_path(name), self.path_include)
	
	##
	## @brief Get the path where to generate files when needed (before compiling / installing it)
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @return (string) The path
	##
	def get_build_path_temporary_generate(self, name):
		return os.path.join(self.get_build_path(name), self.path_temporary_generate)
	
	##
	## @brief Get the path filename of the build binary name
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @return (string) The path
	##
	def get_build_file_bin(self, name, static):
		if static == True:
			return os.path.join(self.get_build_path_bin(name), name + "_static" + self.suffix_binary)
		return os.path.join(self.get_build_path_bin(name), name + "_dynamic" + self.suffix_binary)
	
	##
	## @brief Get the path filename of the build static library name
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @return (string) The path
	##
	def get_build_file_static(self, name):
		return os.path.join(self.get_build_path_lib(name), self.prefix_lib + name + self.suffix_lib_static)
	
	##
	## @brief Get the path filename of the build shared library name
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @return (string) The path
	##
	def get_build_file_dynamic(self, name):
		return os.path.join(self.get_build_path_lib(name), self.prefix_lib + name + self.suffix_lib_dynamic)
	
	
	##
	## @brief Get the bin path for staging step
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the package
	## @return (string) The path
	##
	def get_staging_path_bin(self, name, tmp=False):
		return os.path.join(self.get_staging_path(name, tmp), self.path_bin)
	
	##
	## @brief Get the lib path for staging step
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the package
	## @return (string) The path
	##
	def get_staging_path_lib(self, name, tmp=False):
		return os.path.join(self.get_staging_path(name, tmp), self.path_lib, name)
	
	##
	## @brief Get the data path for staging step
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the package
	## @return (string) The path
	##
	def get_staging_path_data(self, name, tmp=False):
		return os.path.join(self.get_staging_path(name, tmp), self.path_data, name)
	
	##
	## @brief Get the include path for staging step
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the package
	## @return (string) The path
	##
	def get_staging_path_include(self, name):
		return os.path.join(self.get_staging_path(name), self.path_include)
	
	def get_doc_path(self, module_name):
		return os.path.join(tools.get_run_path(), self.path_out, self.path_doc, module_name)
	
	def is_module_build(self, my_module):
		for mod in self.build_done:
			if mod == my_module:
				return True
		self.build_done.append(my_module)
		return False
	
	def is_module_build_tree(self, my_module):
		for mod in self.build_tree_done:
			if mod == my_module:
				return True
		self.build_tree_done.append(my_module)
		return False
	
	##
	## @brief Add new loaded module
	## @param[in] self (handle) Class handle
	## @param[in] new_module (handle) pointer on the module instance
	## @return None
	##
	def add_module(self, new_module):
		debug.debug("Add nodule for Taget : " + new_module.get_name())
		self.module_list.append(new_module)
	
	##
	## @brief Get a module handle that is used in this target
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @return (handle|None) @ref lutin.module.Module pointer on the module requested or None
	##
	def get_module(self, name):
		for mod in self.module_list:
			if mod.get_name() == name:
				return mod
		debug.error("the module '" + str(name) + "'does not exist/already build")
		return None
	
	##
	## @brief Build data associated at the module name in a specific package
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @param[in] package_name (string) Name of the package
	## @return None
	##
	def build_tree(self, name, package_name):
		for mod in self.module_list:
			if mod.get_name() == name:
				mod.build_tree(self, package_name)
				return
		debug.error("request to build tree on un-existant module name : '" + name + "'")
	
	##
	## @brief Clean a specific module for this target (clean all data in the "out" path)
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module
	## @return None
	##
	def clean(self, name):
		for mod in self.module_list:
			if mod.get_name() == name:
				mod.clean(self)
				return
		debug.error("request to clean an un-existant module name : '" + name + "'")
	
	##
	## @brief Load a specific module if it accessible
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Name of the module to load
	## @param[in] optionnal (bool) not create an error if the module does not exist.
	## @return (bool) module loading status
	##
	def load_if_needed(self, name, optionnal=False):
		for elem in self.module_list:
			if elem.get_name() == name:
				return True
		# try to find in the local Modules:
		exist = module.exist(self, name)
		if exist == True:
			module.load_module(self, name)
			return True;
		# need to import the module (or the system module ...)
		exist = system.exist(name, self._config_based_on, self)
		if exist == True:
			system.load(self, name, self._config_based_on)
			return True;
		# we did not find the module ...
		return False;
	
	##
	## @brief Load all module that are accessible in the worktree
	## @param[in] self (handle) Class handle
	## @return None
	##
	def load_all(self):
		listOfAllTheModule = module.list_all_module()
		for modName in listOfAllTheModule:
			self.load_if_needed(modName)
	
	##
	## @brief Build action on the target (execute specific actions on the modules...)
	## @note Recursive call ...
	## @param[in] self (handle) Class handle
	## @param[in] name (string) Module to build
	## @param[in] optionnal (bool) If the module is not accessible, this is not a probleme ==> optionnal dependency requested
	## @param[in] actions ([string,...]) list of action to do. ex: build, gcov, dump, all, clean, install, uninstall, run, log
	## @param[in] package_name Current package name that request the build
	## @return (None|Module handle| ...) complicated return ...
	##
	def build(self, name, optionnal=False, actions=[], package_name=None):
		if    len(name.split("?")) != 1\
		   or len(name.split("@")) != 1:
			debug.error("need update")
		if actions == "":
			actions = ["build"]
		if actions == []:
			actions = ["build"]
		if type(actions) == str:
			actions = [actions]
		if name == "gcov":
			debug.info("gcov all")
			debug.error("must set the gcov parsing on a specific library or binary ==> not supported now for all")
		if name == "dump":
			debug.info("dump all")
			self.load_all()
			for mod in self.module_list:
				mod.display()
			return
		if name[:10] == "dependency":
			if len(name) > 10:
				rules = name.split(":")[1]
			else:
				rules = "LBDPK"
				# L for library
				# B for binary
				# D for Data
				# P for prebuild
				# K for package
			debug.print_element("dot", "", "---", "dependency.dot")
			self.load_all()
			tmp_file = open("dependency.dot", 'w')
			tmp_file.write('digraph G {\n')
			tmp_file.write('	rankdir=\"LR\";\n')
			for mod in self.module_list:
				mod.dependency_generate(self, tmp_file, 1, rules)
			# TODO : do it better ==> system library hook (do a oad of all avillable system library)
			tmp_file.write('	node [\n');
			tmp_file.write('		shape=square;\n');
			tmp_file.write('		style=filled;\n');
			tmp_file.write('		color=gray;\n');
			tmp_file.write('		];\n');
			# TODO : End hook
			for mod in self.module_list:
				mod.dependency_generate(self, tmp_file, 2, rules)
			tmp_file.write('}\n')
			tmp_file.flush()
			tmp_file.close()
			debug.print_element("dot", "", "---", "dependency.dot")
			return
		if name == "all":
			debug.info("build all")
			self.load_all()
			for mod in self.module_list:
				if self._name == "Android":
					if mod.get_type() == "PACKAGE":
						mod.build(self, package_name)
				else:
					if    mod.get_type() == "BINARY" \
					   or mod.get_type() == "PACKAGE":
						mod.build(self, package_name)
		elif name == "clean":
			debug.info("clean all")
			self.load_all()
			for mod in self.module_list:
				mod.clean(self)
		else:
			module_name = name
			action_list = actions
			for action_name in action_list:
				debug.verbose("requested : " + module_name + "?" + action_name + " [START]")
				ret = None;
				if action_name == "install":
					try:
						self.install_package(module_name)
					except AttributeError:
						debug.error("target have no 'install_package' instruction")
				elif action_name == "uninstall":
					try:
						self.un_install_package(module_name)
					except AttributeError:
						debug.error("target have no 'un_install_package' instruction")
				elif action_name[:3] == "run":
					"""
					if     mod.get_type() != "BINARY" \
					   and mod.get_type() != "PACKAGE":
						debug.error("Can not run other than 'BINARY' ... pakage='" + mod.get_type() + "' for module='" + module_name + "'")
					"""
					bin_name = None
					if len(action_name) > 3:
						if action_name[3] == '%':
							bin_name = ""
							for elem in action_name[4:]:
								if elem == ":":
									break;
								bin_name += elem
						# we have option:
						action_name2 = action_name.replace("\:", "1234COLUMN4321")
						option_list = action_name2.split(":")
						if len(option_list) == 0:
							if bin_name != None:
								debug.warning("action 'run' wrong options options ... : '" + action_name + "' might be separate with ':'")
							option_list = []
						else:
							option_list_tmp = option_list[1:]
							option_list = []
							for elem in option_list_tmp:
								option_list.append(elem.replace("1234COLUMN4321", ":"))
					else:
						option_list = []
					#try:
					self.run(module_name, option_list, bin_name)
					#except AttributeError:
					#	debug.error("target have no 'run' instruction")
				elif action_name == "log":
					try:
						self.show_log(module_name)
					except AttributeError:
						debug.error("target have no 'show_log' instruction")
				else:
					present = self.load_if_needed(module_name, optionnal=optionnal)
					if     present == False \
					   and optionnal == True:
						ret = [heritage.HeritageList(), False]
					else:
						for mod in self.module_list:
							debug.verbose("compare " + mod.get_name() + " == " + module_name)
							if mod.get_name() == module_name:
								if action_name[:4] == "dump":
									debug.info("dump module '" + module_name + "'")
									if len(action_name) > 4:
										debug.warning("action 'dump' does not support options ... : '" + action_name + "'")
									ret = mod.display()
									break
								elif action_name[:5] == "clean":
									debug.info("clean module '" + module_name + "'")
									if len(action_name) > 5:
										debug.warning("action 'clean' does not support options ... : '" + action_name + "'")
									ret = mod.clean(self)
									break
								elif action_name[:4] == "gcov":
									debug.debug("gcov on module '" + module_name + "'")
									if len(action_name) > 4:
										# we have option:
										option_list = action_name.split(":")
										if len(option_list) == 0:
											debug.warning("action 'gcov' wrong options options ... : '" + action_name + "' might be separate with ':'")
											option_list = []
										else:
											option_list = option_list[1:]
									else:
										option_list = []
									if "output" in option_list:
										ret = mod.gcov(self, generate_output=True)
									else:
										ret = mod.gcov(self, generate_output=False)
									break
								elif action_name[:5] == "build":
									if len(action_name) > 5:
										debug.warning("action 'build' does not support options ... : '" + action_name + "'")
									debug.debug("build module '" + module_name + "'")
									if optionnal == True:
										ret = [mod.build(self, package_name), True]
									else:
										ret = mod.build(self, package_name)
									break
						if     optionnal == True \
						   and ret == None:
							ret = [heritage.HeritageList(), False]
							break
						if ret == None:
							debug.error("not know module name : '" + module_name + "' to '" + action_name + "' it")
				debug.verbose("requested : " + module_name + "?" + action_name + " [STOP]")
			if len(action_list) == 1:
				return ret
	
	##
	## @brief Add action to do for package specific part when build upper element
	## @param[in] name_of_state (string) a state to call action
	##     - BINARY
	##     - BINARY_SHARED
	##     - BINARY_STAND_ALONE
	##     - LIBRARY
	##     - LIBRARY_DYNAMIC
	##     - LIBRARY_STATIC
	##     - PACKAGE
	##     - PREBUILD
	##     - DATA
	## @param[in] level (int) Value order to apply action
	## @param[in] name (string) Name of the action
	## @param[in] action (function handle) Function to call to execure action
	## @return None
	##
	def add_action(self, name_of_state="PACKAGE", level=5, name="no-name", action=None):
		debug.verbose("add action : " + name)
		if name_of_state not in self.action_on_state:
			self.action_on_state[name_of_state] = [[level, name, action]]
		else:
			self.action_on_state[name_of_state].append([level, name, action])
	
	##
	## @brief Create a package/bundle for the platform.
	## @param[in] pkg_name Package Name (generic name)
	## @param[in] pkg_properties Property of the package
	## @param[in] base_pkg_path Base path of the package
	## @param[in] heritage_list List of dependency of the package
	## @param[in] static The package is build in static mode
	##
	def make_package(self, pkg_name, pkg_properties, base_pkg_path, heritage_list):
		debug.debug("make_package [START]")
		#The package generated depend of the type of the element:
		end_point_module_name = heritage_list.list_heritage[-1].name
		module = self.get_module(end_point_module_name)
		if module == None:
			debug.error("can not create package ... ");
		if module.get_type() == 'PREBUILD':
			#nothing to do ...
			return
		elif    module.get_type() == 'LIBRARY' \
		     or module.get_type() == 'LIBRARY_DYNAMIC' \
		     or module.get_type() == 'LIBRARY_STATIC':
			debug.info("Can not create package for library");
			return
		elif    module.get_type() == 'BINARY' \
		     or module.get_type() == 'BINARY_STAND_ALONE':
			self.make_package_binary(pkg_name, pkg_properties, base_pkg_path, heritage_list, static = True)
		elif module.get_type() == 'BINARY_SHARED':
			self.make_package_binary(pkg_name, pkg_properties, base_pkg_path, heritage_list, static = False)
		elif module.get_type() == 'PACKAGE':
			self.make_package_binary(pkg_name, pkg_properties, base_pkg_path, heritage_list, static = False)
		debug.debug("make_package [STOP]")
		return
	
	##
	## @brief Create a generic tree of the shared data for each platform
	## @param[in] path_package Path of the basic install folder of the application
	## @param[in] pkg_name Package Name (generic name)
	## @param[in] heritage_list List of dependency of the package
	## @param[in] static The package is build in static mode
	## @return True Something has been copied
	## @return False Nothing has been copied
	##
	def make_package_binary_data(self, path_package, pkg_name, base_pkg_path, heritage_list, static):
		debug.debug("make_package_binary_data [START]")
		target_shared_path = os.path.join(path_package, self.pkg_path_data)
		if static == True:
			path_package_data = os.path.join(target_shared_path, pkg_name)
		else:
			path_package_data = target_shared_path
		tools.create_directory_of_file(path_package_data)
		# prepare list of copy files
		copy_list={}
		debug.debug("heritage for " + str(pkg_name) + ":")
		for heritage in heritage_list.list_heritage:
			debug.debug("sub elements: " + str(heritage.name))
			path_src = self.get_build_path_data(heritage.name)
			debug.verbose("      has directory: " + path_src)
			if os.path.isdir(path_src):
				if static == True:
					debug.debug("      need copy: " + path_src + " to " + path_package_data)
					#copy all data:
					tools.copy_anything(path_src,
					                    path_package_data,
					                    recursive=True,
					                    force_identical=True,
					                    in_list=copy_list)
				else:
					debug.debug("      need copy: " + os.path.dirname(path_src) + " to " + path_package_data)
					#copy all data:
					tools.copy_anything(os.path.dirname(path_src),
					                    path_package_data,
					                    recursive=True,
					                    force_identical=True,
					                    in_list=copy_list)
		#real copy files
		ret_copy = tools.copy_list(copy_list)
		# remove unneded files (NOT folder ...)
		ret_remove = tools.clean_directory(target_shared_path, copy_list)
		debug.debug("make_package_binary_data [STOP]")
		return ret_copy or ret_remove
	
	##
	## @brief Create a generic tree of the binary folder
	## @param[in] path_package Path of the basic install folder of the application
	## @param[in] pkg_name Package Name (generic name)
	## @param[in] heritage_list List of dependency of the package
	## @param[in] static The package is build in static mode
	## @return True Something has been copied
	## @return False Nothing has been copied
	##
	def make_package_binary_bin(self, path_package, pkg_name, base_pkg_path, heritage_list, static):
		debug.debug("make_package_binary_bin [START]")
		copy_list={}
		# creata basic output path
		path_package_bin = os.path.join(path_package, self.pkg_path_bin)
		tools.create_directory_of_file(path_package_bin)
		# Local module binary
		path_src = self.get_build_file_bin(pkg_name, static)
		if os.path.exists(path_src) == True:
			try:
				path_dst = os.path.join(path_package_bin, pkg_name + self.suffix_binary)
				debug.verbose("path_dst: " + str(path_dst))
				tools.copy_file(path_src,
				                path_dst,
				                in_list=copy_list)
			except:
				debug.extreme_verbose("can not find : " + path_src)
				pass
		path_src = self.get_build_file_bin(pkg_name, static)
		path_src = path_src[:len(path_src)-4] + "js"
		if os.path.exists(path_src) == True:
			try:
				path_dst = os.path.join(path_package_bin, pkg_name + self.suffix_binary2)
				debug.verbose("path_dst: " + str(path_dst))
				tools.copy_file(path_src,
				                path_dst,
				                in_list=copy_list)
			except:
				debug.extreme_verbose("can not find : " + path_src)
				pass
		# heritage binary
		debug.debug("heritage for " + str(pkg_name) + ":")
		for heritage in heritage_list.list_heritage:
			debug.debug("sub elements: " + str(heritage.name))
			path_src = self.get_build_file_bin(heritage.name, static)
			if os.path.exists(path_src) == True:
				try:
					path_dst = os.path.join(path_package_bin, heritage.name + self.suffix_binary)
					debug.verbose("path_dst: " + str(path_dst))
					tools.copy_file(path_src,
					                path_dst,
					                in_list=copy_list)
				except:
					debug.extreme_verbose("can not find : " + path_src)
					pass
			path_src = self.get_build_file_bin(heritage.name, static)
			path_src = path_src[:len(path_src)-4] + "js"
			if os.path.exists(path_src) == True:
				try:
					path_dst = os.path.join(path_package_bin, heritage.name + self.suffix_binary2)
					debug.verbose("path_dst: " + str(path_dst))
					tools.copy_file(path_src,
					                path_dst,
					                in_list=copy_list)
				except:
					debug.extreme_verbose("can not find : " + path_src)
					pass
		#real copy files
		ret_copy = tools.copy_list(copy_list)
		ret_remove = False
		if self.pkg_path_bin != "":
			# remove unneded files (NOT folder ...)
			ret_remove = tools.clean_directory(path_package_bin, copy_list)
		debug.debug("make_package_binary_bin [STOP]")
		return ret_copy or ret_remove
	
	##
	## @brief Create a generic tree of the library folder
	## @param[in] path_package Path of the basic install folder of the application
	## @param[in] pkg_name Package Name (generic name)
	## @param[in] heritage_list List of dependency of the package
	## @param[in] static The package is build in static mode
	## @return True Something has been copied
	## @return False Nothing has been copied
	##
	def make_package_binary_lib(self, path_package, pkg_name, base_pkg_path, heritage_list, static):
		debug.debug("make_package_binary_lib [START]")
		copy_list={}
		path_package_lib = os.path.join(path_package, self.pkg_path_lib)
		if static == False:
			#copy all shred libs...
			tools.create_directory_of_file(path_package_lib)
			debug.verbose("libs for " + str(pkg_name) + ":")
			for heritage in heritage_list.list_heritage:
				debug.debug("sub elements: " + str(heritage.name))
				file_src = self.get_build_file_dynamic(heritage.name)
				debug.verbose("      has directory: " + file_src)
				if os.path.isfile(file_src):
					debug.debug("      need copy: " + file_src + " to " + path_package_lib)
					#copy all data:
					# TODO : We can have a problem when writing over library files ...
					tools.copy_file(file_src,
					                os.path.join(path_package_lib, os.path.basename(file_src)),
					                in_list=copy_list)
		#real copy files
		ret_copy = tools.copy_list(copy_list)
		ret_remove = False
		if self.pkg_path_lib != "":
			# remove unneded files (NOT folder ...)
			ret_remove = tools.clean_directory(path_package_lib, copy_list)
		debug.debug("make_package_binary_lib [STOP]")
		return ret_copy or ret_remove
	
	
	def make_package_generic_files(self, path_package, pkg_properties, pkg_name, base_pkg_path, heritage_list, static):
		debug.debug("make_package_generic_files [START]")
		## Create version file:
		ret_version = tools.file_write_data(os.path.join(path_package, self.pkg_path_version_file),
		                                    tools.version_to_string(pkg_properties["VERSION"]),
		                                    only_if_new=True)
		
		## Create maintainer file:
		ret_maintainer = tools.file_write_data(os.path.join(path_package, self.pkg_path_maintainer_file),
		                                       self.generate_list_separate_coma(pkg_properties["MAINTAINER"]),
		                                       only_if_new=True)
		
		## Create appl_name file:
		ret_appl_name = tools.file_write_data(os.path.join(path_package, self.pkg_path_application_name_file),
		                                      "en_EN:" + pkg_properties["NAME"],
		                                      only_if_new=True)
		
		## Create appl_description file:
		ret_appl_desc = tools.file_write_data(os.path.join(path_package, self.pkg_path_application_description_file),
		                                      "en_EN:" + pkg_properties["DESCRIPTION"],
		                                      only_if_new=True)
		
		## Create Readme file:
		readme_file_dest = os.path.join(path_package, self.pkg_path_readme_file)
		ret_readme = False
		if os.path.exists(os.path.join(base_pkg_path, "os-Linux/README"))==True:
			ret_readme = tools.copy_file(os.path.join(base_pkg_path, "os-Linux/README"), readme_file_dest)
		elif os.path.exists(os.path.join(base_pkg_path, "README"))==True:
			ret_readme = tools.copy_file(os.path.join(base_pkg_path, "README"), readme_file_dest)
		elif os.path.exists(os.path.join(base_pkg_path, "README.md"))==True:
			ret_readme = tools.copy_file(os.path.join(base_pkg_path, "README.md"), readme_file_dest)
		else:
			debug.debug("no file 'README', 'README.md' or 'os-Linux/README' ==> generate an empty one")
			ret_readme = tools.file_write_data(readme_file_dest,
			                                   "No documentation for " + pkg_name + "\n",
			                                   only_if_new=True)
		
		## Create licence file:
		"""
		# TODO ...
		license_file_dest = os.path.join(path_package, self.pkg_path_license, pkg_name + ".txt")
		tools.create_directory_of_file(license_file_dest)
		if os.path.exists(base_pkg_path + "/license.txt")==True:
			tools.copy_file(base_pkg_path + "/license.txt", license_file_dest)
		else:
			debug.info("no file 'license.txt' ==> generate an empty one")
			tmpFile = open(license_file_dest, 'w')
			tools.file_write_data(license_file_dest,
			                      "No license define by the developper for " + pkg_name + "\n",
			                      only_if_new=True)
		"""
		
		## Create changeLog file:
		change_log_file_dest = os.path.join(path_package, self.pkg_path_change_log_file)
		ret_changelog = False
		if os.path.exists(os.path.join(base_pkg_path, "changelog")) == True:
			ret_changelog = tools.copy_file(os.path.join(base_pkg_path, "changelog"), change_log_file_dest)
		else:
			debug.debug("no file 'changelog' ==> generate an empty one")
			ret_changelog = tools.file_write_data(change_log_file_dest,
			                                      "No changelog data " + pkg_name + "\n",
			                                      only_if_new=True)
		debug.debug("make_package_generic_files [STOP]")
		return    ret_version \
		       or ret_maintainer \
		       or ret_appl_name \
		       or ret_appl_desc \
		       or ret_readme \
		       or ret_changelog
	
	def install_package(self, pkg_name):
		debug.debug("------------------------------------------------------------------------")
		debug.info("-- Install package '" + pkg_name + "'")
		debug.debug("------------------------------------------------------------------------")
		debug.error("action not implemented ...")
	
	def un_install_package(self, pkg_name):
		debug.debug("------------------------------------------------------------------------")
		debug.info("-- Un-Install package '" + pkg_name + "'")
		debug.debug("------------------------------------------------------------------------")
		debug.error("action not implemented ...")
	
	def run(self, pkg_name, option_list, binary_name = None):
		debug.debug("------------------------------------------------------------------------")
		debug.info("-- Run package '" + pkg_name + "' + option: " + str(option_list))
		debug.debug("------------------------------------------------------------------------")
		debug.error("action not implemented ...")
	
	def show_log(self, pkg_name):
		debug.debug("------------------------------------------------------------------------")
		debug.info("-- Show log logcat '" + pkg_name + "'")
		debug.debug("------------------------------------------------------------------------")
		debug.error("action not implemented ...")
	
	##
	## @brief convert a s list of string in a string separated by a ","
	## @param[in] list List of element to transform
	## @return The requested string
	##
	def generate_list_separate_coma(self, list):
		result = ""
		fistTime = True
		for elem in list:
			if fistTime == True:
				fistTime = False
			else:
				result += ","
			result += elem
		return result


__target_list=[]
__start_target_name="Target_"

##
## @brief Import all File that start with env.get_build_system_base_name() + __start_target_name + XXX and register in the list of Target
## @param[in] path_list ([string,...]) List of file that start with env.get_build_system_base_name() in the running worktree (Parse one time ==> faster)
##
def import_path(path_list):
	global __target_list
	global_base = env.get_build_system_base_name()
	debug.debug("TARGET: Init with Files list:")
	for elem in path_list:
		sys.path.append(os.path.dirname(elem))
		# Get file name:
		filename = os.path.basename(elem)
		# Remove .py at the end:
		filename = filename[:-3]
		# Remove global base name:
		filename = filename[len(global_base):]
		# Check if it start with the local patern:
		if filename[:len(__start_target_name)] != __start_target_name:
			debug.extreme_verbose("TARGET:     NOT-Integrate: '" + filename + "' from '" + elem + "' ==> rejected")
			continue
		# Remove local patern
		target_name = filename[len(__start_target_name):]
		debug.verbose("TARGET:     Integrate: '" + target_name + "' from '" + elem + "'")
		__target_list.append([target_name, elem])
	debug.verbose("New list TARGET: ")
	for elem in __target_list:
		debug.verbose("    " + str(elem[0]))

##
## @brief Load a specific target
##
def load_target(name, config):
	global __target_list
	debug.debug("load target: " + name)
	if len(__target_list) == 0:
		debug.error("No target to compile !!!")
	debug.debug("list target: " + str(__target_list))
	for mod in __target_list:
		if mod[0] == name:
			debug.verbose("add to path: '" + os.path.dirname(mod[1]) + "'")
			sys.path.append(os.path.dirname(mod[1]))
			debug.verbose("import target : '" + env.get_build_system_base_name() + __start_target_name + name + "'")
			theTarget = __import__(env.get_build_system_base_name() + __start_target_name + name)
			#create the target
			tmpTarget = theTarget.Target(config)
			return tmpTarget
	raise KeyError("No entry for : " + name)

def list_all_target():
	global __target_list
	tmpListName = []
	for mod in __target_list:
		tmpListName.append(mod[0])
	return tmpListName

def list_all_target_with_desc():
	global __target_list
	tmpList = []
	for mod in __target_list:
		sys.path.append(os.path.dirname(mod[1]))
		theTarget = __import__(env.get_build_system_base_name() + __start_target_name + mod[0])
		try:
			tmpdesc = theTarget.get_desc()
			tmpList.append([mod[0], tmpdesc])
		except:
			debug.warning("has no name : " + mod[0])
			tmpList.append([mod[0], ""])
	return tmpList



#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

import sys
import os
import inspect
import fnmatch
import datetime
# Local import
from realog import debug
from . import tools
from . import env


__macro_list=[]
__start_macro_name="Macro_"

##
## @brief Import all File that start with env.get_build_system_base_name() + __start_macro_name + XXX and register in the list of Target
## @param[in] path_list ([string,...]) List of file that start with env.get_build_system_base_name() in the running worktree (Parse one time ==> faster)
##
def import_path(path_list):
	global __macro_list
	global_base = env.get_build_system_base_name()
	debug.debug("TARGET: Init with Files list:")
	for elem in path_list:
		sys.path.append(os.path.dirname(elem))
		# Get file name:
		filename = os.path.basename(elem)
		# Remove .py at the end:
		filename = filename[:-3]
		# Remove global base name:
		filename = filename[len(global_base):]
		# Check if it start with the local patern:
		if filename[:len(__start_macro_name)] != __start_macro_name:
			debug.extreme_verbose("MACRO:     NOT-Integrate: '" + filename + "' from '" + elem + "' ==> rejected")
			continue
		# Remove local patern
		target_name = filename[len(__start_macro_name):]
		debug.verbose("MACRO:     Integrate: '" + target_name + "' from '" + elem + "'")
		__macro_list.append([target_name, elem])
	debug.verbose("New list MACRO: ")
	for elem in __macro_list:
		debug.verbose("    " + str(elem[0]))

##
## @brief Load a specific target
##
def load_macro(name):
	global __macro_list
	debug.debug("load macro: " + name)
	if len(__macro_list) == 0:
		debug.error("No macro to compile !!!")
	debug.debug("list macro: " + str(__macro_list))
	for mod in __macro_list:
		if mod[0] == name:
			debug.verbose("add to path: '" + os.path.dirname(mod[1]) + "'")
			sys.path.append(os.path.dirname(mod[1]))
			debug.verbose("import macro : '" + env.get_build_system_base_name() + __start_macro_name + name + "'")
			the_macro = __import__(env.get_build_system_base_name() + __start_macro_name + name)
			return the_macro
	raise KeyError("No entry for : " + name)

def list_all_macro():
	global __macro_list
	tmp_list_name = []
	for mod in __macro_list:
		tmp_list_name.append(mod[0])
	return tmp_list_name



#!/usr/bin/python
# -*- coding: utf-8 -*-
##
## @author Edouard DUPIN
##
## @copyright 2012, Edouard DUPIN, all right reserved
##
## @license MPL v2.0 (see license file)
##

from realog import debug
from lutin import target
from lutin import tools
import os
import stat
import re
from lutin import depend
from lutin import host
from lutin import multiprocess
import lutinTarget_Linux

class Target(lutinTarget_Linux.Target):
	def __init__(self, config, sub_name=[]):
		lutinTarget_Linux.Target.__init__(self, config, ["Debian"] + sub_name)
	
	"""
	.local/application
	    *--> applName -> applName.app/bin/applName
	    *--> applName.app
	        *--> appl_description.txt
	        *--> appl_name.txt
	        *--> changelog.txt
	        *--> copyright.txt
	        *--> readme.txt
	        *--> version.txt
	        *--> website.txt
	        *--> icon.png
	        *--> bin
	        *    *--> applName
	        *--> doc
	        *    *--> applName
	        *--> lib
	        *    *--> XX.so
	        *    *--> YY.so
	        *--> license
	        *    *--> applName.txt
	        *    *--> libXX.txt
	        *    *--> libYY.txt
	        *--> man
	        *--> share
	        *    *--> applName
	        *    *--> XX
	        *    *--> YY
	        *--> sources
	"""
	def make_package_binary(self, pkg_name, pkg_properties, base_pkg_path, heritage_list, static):
		# http://alp.developpez.com/tutoriels/debian/creer-paquet/
		debianpkg_name = re.sub("_", "-", pkg_name)
		debug.debug("------------------------------------------------------------------------")
		debug.info("Generate generic '" + pkg_name + "' v" + tools.version_to_string(pkg_properties["VERSION"]))
		debug.debug("------------------------------------------------------------------------")
		#output path
		target_outpath = os.path.join(self.get_staging_path(pkg_name), pkg_name + ".app")
		tools.create_directory_of_file(target_outpath)
		
		## Create share datas:
		ret_share = self.make_package_binary_data(target_outpath, pkg_name, base_pkg_path, heritage_list, static)
		
		## copy binary files:
		ret_bin = self.make_package_binary_bin(target_outpath, pkg_name, base_pkg_path, heritage_list, static)
		
		## Create libraries:
		ret_lib = self.make_package_binary_lib(target_outpath, pkg_name, base_pkg_path, heritage_list, static)
		
		## Create generic files:
		ret_file = self.make_package_generic_files(target_outpath, pkg_properties, pkg_name, base_pkg_path, heritage_list, static)
		
		## end of the package generation
		build_package_path_done = os.path.join(self.get_build_path(pkg_name), "generatePackageDone.txt")
		#Check date between the current file "list of action to generate package and the end of package generation
		need_generate_package = depend.need_re_package(build_package_path_done, [__file__], True)
		
		## create the package:
		if    ret_share \
		   or ret_bin \
		   or ret_lib \
		   or ret_file \
		   or need_generate_package:
			"""
			## create the package:
			debug.debug("package : " + self.get_staging_path(pkg_name) + "/" + pkg_name + ".app.pkg")
			os.system("cd " + self.get_staging_path(pkg_name) + " ; tar -czf " + pkg_name + ".app.tar.gz " + pkg_name + ".app")
			#multiprocess.run_command("cd " + self.get_staging_path(pkg_name) + " ; tar -czf " + pkg_name + ".app.tar.gz " + pkg_name + ".app")
			tools.create_directory_of_file(self.get_final_path())
			tools.copy_file(self.get_staging_path(pkg_name) + "/" + pkg_name + ".app.tar.gz", self.get_final_path() + "/" + pkg_name + ".app.gpkg")
			def make_package_binary(self, pkg_name, pkg_properties, base_pkg_path, heritage_list, static):
			"""
			self.get_staging_path(pkg_name)
			target_outpathDebian = self.get_staging_path(pkg_name) + "/DEBIAN/"
			finalFileControl = target_outpathDebian + "control"
			finalFilepostRm = target_outpathDebian + "postrm"
			# create the paths :
			tools.create_directory_of_file(finalFileControl)
			tools.create_directory_of_file(finalFilepostRm)
			## Create the control file
			tools.create_directory_of_file(finalFileControl)
			tmpFile = open(finalFileControl, 'w')
			tmpFile.write("Package: " + debianpkg_name + "\n")
			tmpFile.write("Version: " + tools.version_to_string(pkg_properties["VERSION"]) + "\n")
			tmpFile.write("Section: " + self.generate_list_separate_coma(pkg_properties["SECTION"]) + "\n")
			tmpFile.write("Priority: " + pkg_properties["PRIORITY"] + "\n")
			tmpFile.write("Architecture: all\n")
			tmpFile.write("Depends: bash\n")
			tmpFile.write("Maintainer: " + self.generate_list_separate_coma(pkg_properties["MAINTAINER"]) + "\n")
			tmpFile.write("Description: " + pkg_properties["DESCRIPTION"] + "\n")
			tmpFile.write("\n")
			tmpFile.flush()
			tmpFile.close()
			## Create the PostRm
			tmpFile = open(finalFilepostRm, 'w')
			tmpFile.write("#!/bin/bash\n")
			tmpFile.write("touch ~/." + pkg_name + "\n")
			if pkg_name != "":
				tmpFile.write("touch ~/.local/share/" + pkg_name + "\n")
				tmpFile.write("rm -r ~/.local/share/" + pkg_name + "\n")
			tmpFile.write("\n")
			tmpFile.flush()
			tmpFile.close()
			## Enable Execution in script
			os.chmod(finalFilepostRm, stat.S_IRWXU + stat.S_IRGRP + stat.S_IXGRP + stat.S_IROTH + stat.S_IXOTH);
			## Readme donumentation
			readmeFileDest = self.get_staging_path(pkg_name) + "/usr/share/doc/"+ debianpkg_name + "/README"
			tools.create_directory_of_file(readmeFileDest)
			if os.path.exists(base_pkg_path + "/os-Linux/README")==True:
				tools.copy_file(base_pkg_path + "/os-Linux/README", readmeFileDest)
			elif os.path.exists(base_pkg_path + "/README")==True:
				tools.copy_file(base_pkg_path + "/README", readmeFileDest)
			elif os.path.exists(base_pkg_path + "/README.md")==True:
				tools.copy_file(base_pkg_path + "/README.md", readmeFileDest)
			else:
				debug.info("no file 'README', 'README.md' or 'os-Linux/README' ==> generate an empty one")
				tmpFile = open(readmeFileDest, 'w')
				tmpFile.write("No documentation for " + pkg_name + "\n")
				tmpFile.flush()
				tmpFile.close()
			## licence file
			license_file_dest = self.get_staging_path(pkg_name) + "/usr/share/doc/"+ debianpkg_name + "/copyright"
			tools.create_directory_of_file(license_file_dest)
			if os.path.exists(base_pkg_path + "/license.txt")==True:
				tools.copy_file(base_pkg_path + "/license.txt", license_file_dest)
			else:
				debug.info("no file 'license.txt' ==> generate an empty one")
				tmpFile = open(license_file_dest, 'w')
				tmpFile.write("No license define by the developper for " + pkg_name + "\n")
				tmpFile.flush()
				tmpFile.close()
			##changeLog file
			change_log_file_dest = self.get_staging_path(pkg_name) + "/usr/share/doc/"+ debianpkg_name + "/changelog"
			tools.create_directory_of_file(change_log_file_dest)
			if os.path.exists(base_pkg_path + "/changelog")==True:
				tools.copy_file(base_pkg_path + "/changelog", change_log_file_dest)
			else:
				debug.info("no file 'changelog' ==> generate an empty one")
				tmpFile = open(change_log_file_dest, 'w')
				tmpFile.write("No changelog data " + pkg_name + "\n")
				tmpFile.flush()
				tmpFile.close()
			## create the package :
			debug.debug("package : " + self.get_staging_path(pkg_name) + "/" + debianpkg_name + ".deb")
			os.system("cd " + self.get_staging_path("") + " ; dpkg-deb --build " + pkg_name)
			tools.create_directory_of_file(self.get_final_path())
			tools.copy_file(self.get_staging_path("") + "/" + pkg_name + self.suffix_package, self.get_final_path() + "/" + pkg_name + self.suffix_package)
			
			# package is done corectly ...
			tools.file_write_data(build_package_path_done, "done...")
	
	def install_package(self, pkg_name):
		debug.debug("------------------------------------------------------------------------")
		debug.info("Install package '" + pkg_name + "'")
		debug.debug("------------------------------------------------------------------------")
		os.system("sudo dpkg -i " + self.get_final_path() + "/" + pkg_name + self.suffix_package)
	
	def un_install_package(self, pkg_name):
		debug.debug("------------------------------------------------------------------------")
		debug.info("Un-Install package '" + pkg_name + "'")
		debug.debug("------------------------------------------------------------------------")
		os.system("sudo dpkg -r " + self.get_final_path() + "/" + pkg_name + self.suffix_package)
	


